@@ -159,7 +159,7 @@ struct vnodeopv_entry_desc afs_vnodeop_entries[] = {
     {VOPPREF(blktooff_desc), (VOPFUNC)afs_vop_blktooff},	/* blktooff */
     {VOPPREF(offtoblk_desc), (VOPFUNC)afs_vop_offtoblk},	/* offtoblk */
     {VOPPREF(bwrite_desc), (VOPFUNC)vn_bwrite},
-    {(struct vnodeop_desc *)NULL, (void (*)())NULL}
+    {NULL, (void (*)())NULL}
 };
 struct vnodeopv_desc afs_vnodeop_opv_desc =
     { &afs_vnodeop_p, afs_vnodeop_entries };
@@ -202,7 +202,7 @@ struct vnodeopv_entry_desc afs_dead_vnodeop_entries[] = {
     {VOPPREF(blktooff_desc), (VOPFUNC)err_blktooff},	/* blktooff */
     {VOPPREF(offtoblk_desc), (VOPFUNC)err_offtoblk},	/* offtoblk */
     {VOPPREF(bwrite_desc), (VOPFUNC)err_bwrite},
-    {(struct vnodeop_desc *)NULL, (void (*)())NULL}
+    {NULL, (void (*)())NULL}
 };
 struct vnodeopv_desc afs_dead_vnodeop_opv_desc =
     { &afs_dead_vnodeop_p, afs_dead_vnodeop_entries };
@@ -281,7 +281,7 @@ afs_vop_lookup(ap)
     int error;
     struct vcache *vcp;
     struct vnode *vp, *dvp;
-    register int flags = ap->a_cnp->cn_flags;
+    int flags = ap->a_cnp->cn_flags;
     int lockparent;		/* 1 => lockparent flag is set */
     int wantparent;		/* 1 => wantparent or lockparent flag */
     struct proc *p;
@@ -294,7 +294,7 @@ afs_vop_lookup(ap)
      */
     if (ap->a_context == afs_osi_ctxtp)
 	return ENOENT;
-    if (vcp->mvstat != 1) {
+    if (vcp->mvstat != AFS_MVSTAT_MTPT) {
 	error = cache_lookup(ap->a_dvp, ap->a_vpp, ap->a_cnp);
 	if (error == -1) 
 	    return 0;
@@ -402,7 +402,7 @@ afs_vop_create(ap)
 {
     int error = 0;
     struct vcache *vcp;
-    register struct vnode *dvp = ap->a_dvp;
+    struct vnode *dvp = ap->a_dvp;
     struct proc *p;
     GETNAME();
     p = vop_cn_proc;
@@ -530,7 +530,7 @@ afs_vop_close(ap)
 	code = afs_close(avc, ap->a_fflag, vop_cred);
     else
 	code = afs_close(avc, ap->a_fflag, &afs_osi_cred);
-    osi_FlushPages(avc, vop_cred);	/* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(avc, vop_cred);	/* hold GLOCK, but not basic vnode lock */
     /* This is legit; it just forces the fstrace event to happen */
     code = afs_CheckCode(code, NULL, 60);
     AFS_GUNLOCK();
@@ -579,7 +579,7 @@ afs_vop_access(ap)
         code = afs_CheckCode(code, &treq, 56);
         goto out;
     }
-    if (afs_fakestat_enable && tvc->mvstat && !(tvc->f.states & CStatd)) {
+    if (afs_fakestat_enable && tvc->mvstat != AFS_MVSTAT_FILE && !(tvc->f.states & CStatd)) {
         code = 0;
         goto out;
     }
@@ -596,12 +596,6 @@ afs_vop_access(ap)
           bits |= PRSFS_INSERT;
        if (ap->a_action & KAUTH_VNODE_DELETE_CHILD)
           bits |= PRSFS_DELETE;
-#if 0 /* I'd argue this should be enforced on the parent. But that's ugly */
-       if (ap->a_action & KAUTH_VNODE_READ_ATTRIBUTES)
-          bits |= PRSFS_LOOKUP;
-       if (ap->a_action & KAUTH_VNODE_READ_SECURITY) /* mode bits/gid, not afs acl */
-          bits |= PRSFS_LOOKUP;
-#endif
     } else {
        if (ap->a_action & KAUTH_VNODE_READ_DATA)
           bits |= PRSFS_READ;
@@ -622,12 +616,6 @@ afs_vop_access(ap)
     }
     if (ap->a_action & KAUTH_VNODE_WRITE_ATTRIBUTES)
        bits |= PRSFS_WRITE;
-#if 0 /* no extended attributes */
-    if (ap->a_action & KAUTH_VNODE_READ_EXTATTRIBUTES)
-       bits |= PRSFS_READ;
-    if (ap->a_action & KAUTH_VNODE_WRITE_EXTATTRIBUTES)
-       bits |= PRSFS_WRITE;
-#endif
     if (ap->a_action & KAUTH_VNODE_WRITE_SECURITY)
        bits |= PRSFS_WRITE;
     /* we can't check for KAUTH_VNODE_TAKE_OWNERSHIP, so we always permit it */
@@ -700,7 +688,7 @@ afs_vop_getattr(ap)
 	if (!isglock)
 	  AFS_GLOCK();
 	/* do minimal work to return fake result for fsevents */
-	if (afs_fakestat_enable && VTOAFS(ap->a_vp)->mvstat == 1) {
+	if (afs_fakestat_enable && VTOAFS(ap->a_vp)->mvstat == AFS_MVSTAT_MTPT) {
 	    struct afs_fakestat_state fakestat;
 	    struct vrequest treq;
 
@@ -766,16 +754,24 @@ afs_vop_setattr(ap)
 				 * struct proc *a_p;
 				 * } */ *ap;
 {
-    int code;
+    int code, pass = 0;
+    struct vcache *avc = VTOAFS(ap->a_vp);
 #ifdef AFS_DARWIN80_ENV
     /* fsevents tries to set attributes. drop it. */
     if (ap->a_context == afs_osi_ctxtp)
 	return 0;
 #endif
     AFS_GLOCK();
-    code = afs_setattr(VTOAFS(ap->a_vp), ap->a_vap, vop_cred);
+retry:
+    code = afs_setattr(avc, ap->a_vap, vop_cred);
     /* This is legit; it just forces the fstrace event to happen */
     code = afs_CheckCode(code, NULL, 59);
+    if (!pass && code == EINVAL && (VATTR_IS_ACTIVE(ap->a_vap, va_mode) &&
+				    (vType(avc) == VLNK))) {
+	VATTR_CLEAR_ACTIVE(ap->a_vap, va_mode);
+	pass++;
+	goto retry;
+    }
     AFS_GUNLOCK();
     return code;
 }
@@ -803,8 +799,8 @@ afs_vop_read(ap)
     }
 #endif
     AFS_GLOCK();
-    osi_FlushPages(avc, vop_cred);	/* hold bozon lock, but not basic vnode lock */
-    code = afs_read(avc, ap->a_uio, vop_cred, 0, 0, 0);
+    osi_FlushPages(avc, vop_cred);	/* hold GLOCK, but not basic vnode lock */
+    code = afs_read(avc, ap->a_uio, vop_cred, 0);
     AFS_GUNLOCK();
     return code;
 }
@@ -821,7 +817,7 @@ afs_vop_pagein(ap)
 				 * int a_flags;
 				 * } */ *ap;
 {
-    register struct vnode *vp = ap->a_vp;
+    struct vnode *vp = ap->a_vp;
     upl_t pl = ap->a_pl;
     size_t size = ap->a_size;
     off_t f_offset = ap->a_f_offset;
@@ -829,17 +825,20 @@ afs_vop_pagein(ap)
     int flags = ap->a_flags;
     struct ucred *cred;
     vm_offset_t ioaddr;
+    int code;
+    struct vcache *tvc = VTOAFS(vp);
+    int nocommit = flags & UPL_NOCOMMIT;
 #ifdef AFS_DARWIN80_ENV
     struct uio *uio;
 #else
     struct uio auio;
     struct iovec aiov;
     struct uio *uio = &auio;
-#endif
-    int nocommit = flags & UPL_NOCOMMIT;
 
-    int code;
-    struct vcache *tvc = VTOAFS(vp);
+    memset(&auio, 0, sizeof(auio));
+    memset(&aiov, 0, sizeof(aiov));
+#endif
+
 #ifndef AFS_DARWIN80_ENV
     if (UBCINVALID(vp)) {
 #if DIAGNOSTIC
@@ -889,8 +888,8 @@ afs_vop_pagein(ap)
     aiov.iov_base = (caddr_t) ioaddr;
 #endif
     AFS_GLOCK();
-    osi_FlushPages(tvc, vop_cred);	/* hold bozon lock, but not basic vnode lock */
-    code = afs_read(tvc, uio, cred, 0, 0, 0);
+    osi_FlushPages(tvc, vop_cred);	/* hold GLOCK, but not basic vnode lock */
+    code = afs_read(tvc, uio, cred, 0);
     if (code == 0) {
 	ObtainWriteLock(&tvc->lock, 2);
 	tvc->f.states |= CMAPPED;
@@ -947,7 +946,7 @@ afs_vop_write(ap)
 			 AFS_UIO_RESID(ap->a_uio));
 #endif
     AFS_GLOCK();
-    osi_FlushPages(avc, vop_cred);	/* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(avc, vop_cred);	/* hold GLOCK, but not basic vnode lock */
     code =
 	afs_write(VTOAFS(ap->a_vp), ap->a_uio, ap->a_ioflag, vop_cred, 0);
     AFS_GUNLOCK();
@@ -966,7 +965,7 @@ afs_vop_pageout(ap)
 				 * int           a_flags
 				 * } */ *ap;
 {
-    register struct vnode *vp = ap->a_vp;
+    struct vnode *vp = ap->a_vp;
     upl_t pl = ap->a_pl;
     size_t size = ap->a_size;
     off_t f_offset = ap->a_f_offset;
@@ -974,18 +973,21 @@ afs_vop_pageout(ap)
     int flags = ap->a_flags;
     struct ucred *cred;
     vm_offset_t ioaddr;
+    int nocommit = flags & UPL_NOCOMMIT;
+    int iosize;
+    int code;
+    struct vcache *tvc = VTOAFS(vp);
 #ifdef AFS_DARWIN80_ENV
     struct uio *uio;
 #else
     struct uio auio;
     struct iovec aiov;
     struct uio *uio = &auio;
-#endif
-    int nocommit = flags & UPL_NOCOMMIT;
-    int iosize;
 
-    int code;
-    struct vcache *tvc = VTOAFS(vp);
+    memset(&auio, 0, sizeof(auio));
+    memset(&aiov, 0, sizeof(aiov));
+#endif
+
 #ifndef AFS_DARWIN80_ENV
     if (UBCINVALID(vp)) {
 #if DIAGNOSTIC
@@ -1095,7 +1097,7 @@ afs_vop_pageout(ap)
     }
 
     AFS_GLOCK();
-    osi_FlushPages(tvc, vop_cred);	/* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(tvc, vop_cred);	/* hold GLOCK, but not basic vnode lock */
     ObtainWriteLock(&tvc->lock, 1);
     afs_FakeOpen(tvc);
     ReleaseWriteLock(&tvc->lock);
@@ -1199,9 +1201,13 @@ afs_vop_fsync(ap)
 {
     int wait = ap->a_waitfor == MNT_WAIT;
     int error;
-    register struct vnode *vp = ap->a_vp;
+    struct vnode *vp = ap->a_vp;
     int haveGlock = ISAFS_GLOCK();
 
+    /* in order to recycle faked vnodes for bulkstat */
+    if (VTOAFS(vp) == NULL)
+	return ENOTSUP;
+
     /* afs_vop_lookup glocks, can call us through vinvalbuf from GetVCache */
     if (!haveGlock) AFS_GLOCK();
     if (vop_cred)
@@ -1237,8 +1243,8 @@ afs_vop_remove(ap)
 				 * } */ *ap;
 {
     int error = 0;
-    register struct vnode *vp = ap->a_vp;
-    register struct vnode *dvp = ap->a_dvp;
+    struct vnode *vp = ap->a_vp;
+    struct vnode *dvp = ap->a_dvp;
 
 #ifdef AFS_DARWIN80_ENV
     if (ap->a_flags & VNODE_REMOVE_NODELETEBUSY) {
@@ -1302,8 +1308,8 @@ afs_vop_link(ap)
 				 * } */ *ap;
 {
     int error = 0;
-    register struct vnode *dvp = ap->a_tdvp;
-    register struct vnode *vp = ap->a_vp;
+    struct vnode *dvp = ap->a_tdvp;
+    struct vnode *vp = ap->a_vp;
     struct proc *p;
 
     GETNAME();
@@ -1354,9 +1360,9 @@ afs_vop_rename(ap)
     struct componentname *tcnp = ap->a_tcnp;
     char *tname;
     struct vnode *tvp = ap->a_tvp;
-    register struct vnode *tdvp = ap->a_tdvp;
+    struct vnode *tdvp = ap->a_tdvp;
     struct vnode *fvp = ap->a_fvp;
-    register struct vnode *fdvp = ap->a_fdvp;
+    struct vnode *fdvp = ap->a_fdvp;
     struct proc *p; 
 
     p = cn_proc(fcnp);
@@ -1472,8 +1478,8 @@ afs_vop_rename(ap)
 	tvc = VTOAFS(tdvp);
 
         /* unrewritten mount point? */
-        if (tvc->mvstat == 1) {
-            if (tvc->mvid && (tvc->f.states & CMValid)) {
+        if (tvc->mvstat == AFS_MVSTAT_MTPT) {
+            if (tvc->mvid.target_root && (tvc->f.states & CMValid)) {
                 struct vrequest treq;
 
                 afs_InitFakeStat(&fakestate);
@@ -1517,7 +1523,7 @@ afs_vop_rename(ap)
                 afs_osi_Sleep(tb);
             }
             /* if we succeeded, clear the error. otherwise, EXDEV */
-            if (mvReq.retval == 0)
+            if (tb->code_raw == 0)
                 error = 0;
 
             afs_BRelease(tb);
@@ -1554,8 +1560,8 @@ afs_vop_mkdir(ap)
 				 * struct vattr *a_vap;
 				 * } */ *ap;
 {
-    register struct vnode *dvp = ap->a_dvp;
-    register struct vattr *vap = ap->a_vap;
+    struct vnode *dvp = ap->a_dvp;
+    struct vattr *vap = ap->a_vap;
     int error = 0;
     struct vcache *vcp;
     struct proc *p;
@@ -1605,8 +1611,8 @@ afs_vop_rmdir(ap)
 				 * } */ *ap;
 {
     int error = 0;
-    register struct vnode *vp = ap->a_vp;
-    register struct vnode *dvp = ap->a_dvp;
+    struct vnode *vp = ap->a_vp;
+    struct vnode *dvp = ap->a_dvp;
 
     GETNAME();
     if (dvp == vp) {
@@ -1642,20 +1648,26 @@ afs_vop_symlink(ap)
 				 * char *a_target;
 				 * } */ *ap;
 {
-    register struct vnode *dvp = ap->a_dvp;
+    struct vnode *dvp = ap->a_dvp;
+    struct vcache *pvc = NULL;
     int error = 0;
-    /* NFS ignores a_vpp; so do we. */
 
     GETNAME();
     AFS_GLOCK();
-    error =
-	afs_symlink(VTOAFS(dvp), name, ap->a_vap, ap->a_target, vop_cn_cred);
+    error = afs_symlink(VTOAFS(dvp), name, ap->a_vap, ap->a_target, &pvc,
+			vop_cn_cred);
     AFS_GUNLOCK();
-    DROPNAME();
 #ifndef AFS_DARWIN80_ENV
     FREE_ZONE(cnp->cn_pnbuf, cnp->cn_pnlen, M_NAMEI);
     vput(dvp);
 #endif
+    *ap->a_vpp = NULL;
+    if (!error) {
+	error = afs_darwin_finalizevnode(pvc, dvp, ap->a_cnp, 0, 0);
+	if (!error)
+	    *ap->a_vpp = AFSTOV(pvc);
+    }
+    DROPNAME();
     return error;
 }
 
@@ -1742,7 +1754,7 @@ afs_vop_inactive(ap)
 				 * struct proc *a_p;
 				 * } */ *ap;
 {
-    register struct vnode *vp = ap->a_vp;
+    struct vnode *vp = ap->a_vp;
     struct vcache *tvc = VTOAFS(vp);
 #ifndef AFS_DARWIN80_ENV
     if (prtactive && vp->v_usecount != 0)
@@ -1776,7 +1788,7 @@ afs_vop_reclaim(ap)
 {
     int error = 0;
     int sl, writelocked;
-    register struct vnode *vp = ap->a_vp;
+    struct vnode *vp = ap->a_vp;
     struct vcache *tvc = VTOAFS(vp);
 
     osi_Assert(!ISAFS_GLOCK());
@@ -1951,8 +1963,8 @@ afs_vop_lock(ap)
 				 * struct vnode *a_vp;
 				 * } */ *ap;
 {
-    register struct vnode *vp = ap->a_vp;
-    register struct vcache *avc = VTOAFS(vp);
+    struct vnode *vp = ap->a_vp;
+    struct vcache *avc = VTOAFS(vp);
 
     if (vp->v_tag == VT_NON)
 	return (ENOENT);
@@ -1986,7 +1998,7 @@ afs_vop_truncate(ap)
 				 * } */ *ap;
 {
     /* printf("stray afs_vop_truncate\n"); */
-    return EOPNOTSUPP;
+    return ENOTSUP;
 }
 
 int
@@ -1999,7 +2011,7 @@ afs_vop_update(ap)
 				 * } */ *ap;
 {
     /* printf("stray afs_vop_update\n"); */
-    return EOPNOTSUPP;
+    return ENOTSUP;
 }
 
 int
@@ -2022,10 +2034,6 @@ afs_vop_bmap(ap)
     }
     if (ap->a_runp != NULL)
 	*ap->a_runp = 0;
-#ifdef notyet
-    if (ap->a_runb != NULL)
-	*ap->a_runb = 0;
-#endif
 
     return 0;
 }
@@ -2049,8 +2057,8 @@ afs_vop_print(ap)
 				 * struct vnode *a_vp;
 				 * } */ *ap;
 {
-    register struct vnode *vp = ap->a_vp;
-    register struct vcache *vc = VTOAFS(ap->a_vp);
+    struct vnode *vp = ap->a_vp;
+    struct vcache *vc = VTOAFS(ap->a_vp);
     int s = vc->f.states;
     printf("tag %d, fid: %ld.%x.%x.%x, opens %d, writers %d", vp->v_tag,
 	   vc->f.fid.Cell, vc->f.fid.Fid.Volume, vc->f.fid.Fid.Vnode,
@@ -2101,7 +2109,7 @@ afs_vop_cmap(ap)
 #endif
 
 int
-afs_darwin_getnewvnode(struct vcache *avc, int recycle)
+afs_darwin_getnewvnode(struct vcache *avc)
 {
 #ifdef AFS_DARWIN80_ENV
     vnode_t vp;
@@ -2120,8 +2128,7 @@ afs_darwin_getnewvnode(struct vcache *avc, int recycle)
       vnode_addfsref(vp);
       vnode_ref(vp);
       avc->v = vp;
-      if (recycle)
-	  vnode_recycle(vp); /* terminate as soon as iocount drops */
+      vnode_recycle(vp); /* terminate as soon as iocount drops */
       avc->f.states |= CDeadVnode;
     }
     return error;
@@ -2137,35 +2144,33 @@ afs_darwin_getnewvnode(struct vcache *avc, int recycle)
 #ifdef AFS_DARWIN80_ENV
 /* if this fails, then tvc has been unrefed and may have been freed. 
    Don't touch! */
-int 
-afs_darwin_finalizevnode(struct vcache *avc, struct vnode *dvp, struct componentname *cnp, int isroot, int locked)
+int
+afs_darwin_finalizevnode(struct vcache *avc, struct vnode *dvp,
+			 struct componentname *cnp, int isroot, int locked)
 {
     vnode_t ovp;
     vnode_t nvp;
     int error;
     struct vnode_fsparam par;
+
     if (!locked) {
 	AFS_GLOCK();
 	ObtainWriteLock(&avc->lock,325);
     }
     ovp = AFSTOV(avc);
+
     if (!(avc->f.states & CDeadVnode) && vnode_vtype(ovp) != VNON) {
-        AFS_GUNLOCK();
-#if 0 /* unsupported */
-        if (dvp && cnp)
-	    vnode_update_identity(ovp, dvp, cnp->cn_nameptr, cnp->cn_namelen,
-				  cnp->cn_hash,
-				  VNODE_UPDATE_PARENT|VNODE_UPDATE_NAME);
-#endif
+	AFS_GUNLOCK();
 	/* Can end up in reclaim... drop GLOCK */
-        vnode_rele(ovp);
+	vnode_rele(ovp);
 	AFS_GLOCK();
 	if (!locked) {
 	    ReleaseWriteLock(&avc->lock);
 	    AFS_GUNLOCK();
 	}
-        return 0;
+	return 0;
     }
+
     if ((avc->f.states & CDeadVnode) && vnode_vtype(ovp) != VNON)
 	panic("vcache %p should not be CDeadVnode", avc);
     AFS_GUNLOCK();
@@ -2188,17 +2193,41 @@ afs_darwin_finalizevnode(struct vcache *avc, struct vnode *dvp, struct component
 	if ((avc->f.states & CDeadVnode) && vnode_vtype(ovp) != VNON)
 	    printf("vcache %p should not be CDeadVnode", avc);
 	if (avc->v == ovp) {
-	    if (avc->f.states & CBulkFetching) {
-		vnode_recycle(ovp);
-	    }
 	    if (!(avc->f.states & CVInit)) {
 		vnode_clearfsnode(ovp);
 		vnode_removefsref(ovp);
 	    }
+	    /* we're discarding on a fixup. mark for recycle */
+	    if (!(avc->f.states & CDeadVnode))
+		vnode_recycle(ovp);
 	}
 	avc->v = nvp;
 	avc->f.states &=~ CDeadVnode;
+	/* If we were carrying an extra ref for dirty, hold/push it. */
+	if (avc->f.ddirty_flags) {
+	    vnode_get(nvp);
+	    vnode_ref(nvp);
+	}
+	/* If we were carrying an extra ref for shadow, hold/push it. */
+	if (avc->f.shadow.vnode) {
+	    vnode_get(nvp);
+	    vnode_ref(nvp);
+	}
     }
+    /* Drop any extra dirty ref on the old vnode */
+    if (avc->f.ddirty_flags) {
+	vnode_put(ovp);
+	vnode_rele(ovp);
+    }
+    /* Drop any extra shadow ref on the old vnode */
+    if (avc->f.shadow.vnode) {
+	vnode_put(ovp);
+	vnode_rele(ovp);
+    }
+    /* If it's ref'd still, unmark stat'd to force new lookup */
+    if ((vnode_vtype(ovp) != avc->f.m.Type) && VREFCOUNT_GT(avc, 1))
+	avc->f.states &= ~CStatd;
+
     vnode_put(ovp);
     vnode_rele(ovp);
     AFS_GLOCK();
