@@ -32,8 +32,6 @@ mutex_t afs_init_kern_lock;
 
 #define SYS_setgroups SGI_SETGROUPS
 
-int (*nfs_rfsdisptab_v2) () = NULL;
-
 int afs_fstype;
 lock_t afs_rxlock;
 
@@ -186,11 +184,6 @@ afs_mount(struct vfs *afsp, vnode_t * mvp, struct mounta *uap,
     afsp->vfs_fstype = afs_fstype;
     afsp->vfs_dev = 0xbabebabe;	/* XXX this should be unique */
 
-#ifndef	AFS_NONFSTRANS
-    if (nfs_rfsdisptab_v2)
-	afs_xlatorinit_v2(nfs_rfsdisptab_v2);
-    afs_xlatorinit_v3();
-#endif
     return 0;
 }
 
@@ -201,7 +194,7 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
 {
     struct vcache *tvc;
     vnode_t *vp, *rootvp = NULL;
-    register struct afs_q *tq;
+    struct afs_q *tq;
     struct afs_q *uq;
     int error, fv_slept;
     OSI_VFS_CONVERT(afsp);
@@ -216,11 +209,12 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
      * EBUSY if any still in use
      */
     ObtainWriteLock(&afs_xvcache, 172);
+ retry:
     for (tq = VLRU.prev; tq != &VLRU; tq = uq) {
 	tvc = QTOV(tq);
 	uq = QPrev(tq);
 	vp = (vnode_t *) tvc;
-	if (error = afs_FlushVCache(tvc, &fv_slept))
+	if (error = afs_FlushVCache(tvc, &fv_slept)) {
 	    if (vp->v_flag & VROOT) {
 		rootvp = vp;
 		continue;
@@ -228,6 +222,10 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
 		ReleaseWriteLock(&afs_xvcache);
 		return error;
 	    }
+	}
+	if (fv_slept) {
+	    goto retry;
+	}
     }
 
     /*
@@ -260,9 +258,9 @@ afs_root(OSI_VFS_ARG(afsp), avpp)
     OSI_VFS_DECL(afsp);
      struct vnode **avpp;
 {
-    register afs_int32 code = 0;
+    afs_int32 code = 0;
     struct vrequest treq;
-    register struct vcache *tvp = 0;
+    struct vcache *tvp = 0;
     OSI_VFS_CONVERT(afsp);
 
     AFS_STATCNT(afs_root);
@@ -276,12 +274,12 @@ afs_root(OSI_VFS_ARG(afsp), avpp)
 
 	if (!(code = afs_InitReq(&treq, OSI_GET_CURRENT_CRED()))
 	    && !(code = afs_CheckInit())) {
-	    tvp = afs_GetVCache(&afs_rootFid, &treq, NULL, NULL);
+	    tvp = afs_GetVCache(&afs_rootFid, &treq);
 	    /* we really want this to stay around */
 	    if (tvp) {
 		afs_globalVp = tvp;
 	    } else
-		code = ENOENT;
+		code = EIO;
 	}
     }
     if (tvp) {
@@ -315,10 +313,10 @@ afs_statfs(OSI_VFS_ARG(afsp), abp, avp)
      * before storing something there.
      */
     abp->f_blocks = abp->f_bfree = abp->f_bavail = abp->f_files =
-	abp->f_ffree = abp->f_favail = 900000;
+	abp->f_ffree = abp->f_favail = AFS_VFS_FAKEFREE;
 
     abp->f_fsid = AFS_VFSMAGIC;	/* magic */
-    strcpy(abp->f_basetype, AFS_MOUNT_AFS);
+    strcpy(abp->f_basetype, AFS_MOUNT_STR);
     abp->f_flag = 0;
     abp->f_namemax = 256;
     return 0;
@@ -367,7 +365,7 @@ afs_sync(OSI_VFS_DECL(afsp),
     struct vcache *tvc;
     struct vnode *vp;
     afs_uint32 lvcachegen;
-    register struct afs_q *tq;
+    struct afs_q *tq;
     struct afs_q *uq;
     int s;
     OSI_VFS_CONVERT(afsp);
@@ -511,8 +509,8 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t ** avcp, struct fid * fidp)
 {
     struct VenusFid vfid;
     struct vrequest treq;
-    register struct cell *tcell;
-    register afs_int32 code = 0;
+    struct cell *tcell;
+    afs_int32 code = 0;
     afs_int32 ret;
 
 #if defined(AFS_SGI64_ENV) && defined(CKPT) && !defined(_R5000_CVT_WAR)
@@ -531,7 +529,7 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t ** avcp, struct fid * fidp)
 	/* It's a checkpoint restart fid. */
 	tcell = afs_GetCellByIndex(afid2->af_cell, READ_LOCK);
 	if (!tcell) {
-	    code = ENOENT;
+	    code = EIO;
 	    goto out;
 	}
 	vfid.Cell = tcell->cellNum;
@@ -543,9 +541,9 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t ** avcp, struct fid * fidp)
 	if (code = afs_InitReq(&treq, OSI_GET_CURRENT_CRED()))
 	    goto out;
 	*avcp =
-	    (vnode_t *) afs_GetVCache(&vfid, &treq, NULL, (struct vcache *)0);
+	    (vnode_t *) afs_GetVCache(&vfid, &treq);
 	if (!*avcp) {
-	    code = ENOENT;
+	    code = EIO;
 	}
 	goto out;
     }
