@@ -24,8 +24,6 @@
 #include "afs/nfsclient.h"
 #include "afs/afs_osidnlc.h"
 
-extern afs_rwlock_t afs_xcbhash;
-
 /* Note that we don't set CDirty here, this is OK because the link
  * RPC is called synchronously. */
 
@@ -38,33 +36,38 @@ afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname,
 	 afs_ucred_t *acred)
 #endif
 {
-    struct vrequest treq;
-    register struct dcache *tdc;
-    register afs_int32 code;
-    register struct afs_conn *tc;
+    struct vrequest *treq = NULL;
+    struct dcache *tdc;
+    afs_int32 code;
+    struct afs_conn *tc;
     afs_size_t offset, len;
-    struct AFSFetchStatus OutFidStatus, OutDirStatus;
+    struct AFSFetchStatus *OutFidStatus, *OutDirStatus;
     struct AFSVolSync tsync;
     struct afs_fakestat_state vfakestate, dfakestate;
+    struct rx_connection *rxconn;
     XSTATS_DECLS;
     OSI_VC_CONVERT(adp);
 
     AFS_STATCNT(afs_link);
     afs_Trace3(afs_iclSetp, CM_TRACE_LINK, ICL_TYPE_POINTER, adp,
 	       ICL_TYPE_POINTER, avc, ICL_TYPE_STRING, aname);
+
+    OutFidStatus = osi_AllocSmallSpace(sizeof(struct AFSFetchStatus));
+    OutDirStatus = osi_AllocSmallSpace(sizeof(struct AFSFetchStatus));
+
     /* create a hard link; new entry is aname in dir adp */
-    if ((code = afs_InitReq(&treq, acred)))
+    if ((code = afs_CreateReq(&treq, acred)))
 	goto done2;
 
     afs_InitFakeStat(&vfakestate);
     afs_InitFakeStat(&dfakestate);
-    
+
     AFS_DISCON_LOCK();
 
-    code = afs_EvalFakeStat(&avc, &vfakestate, &treq);
+    code = afs_EvalFakeStat(&avc, &vfakestate, treq);
     if (code)
 	goto done;
-    code = afs_EvalFakeStat(&adp, &dfakestate, &treq);
+    code = afs_EvalFakeStat(&adp, &dfakestate, treq);
     if (code)
 	goto done;
 
@@ -77,7 +80,7 @@ afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname,
 	code = ENAMETOOLONG;
 	goto done;
     }
-    code = afs_VerifyVCache(adp, &treq);
+    code = afs_VerifyVCache(adp, treq);
     if (code)
 	goto done;
 
@@ -88,48 +91,44 @@ afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname,
 	code = EROFS;
 	goto done;
     }
-    
+
     if (AFS_IS_DISCONNECTED) {
         code = ENETDOWN;
         goto done;
     }
 
-    tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);	/* test for error below */
+    tdc = afs_GetDCache(adp, (afs_size_t) 0, treq, &offset, &len, 1);	/* test for error below */
     ObtainWriteLock(&adp->lock, 145);
     do {
-	tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
+	tc = afs_Conn(&adp->f.fid, treq, SHARED_LOCK, &rxconn);
 	if (tc) {
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_LINK);
 	    RX_AFS_GUNLOCK();
 	    code =
-		RXAFS_Link(tc->id, (struct AFSFid *)&adp->f.fid.Fid, aname,
-			   (struct AFSFid *)&avc->f.fid.Fid, &OutFidStatus,
-			   &OutDirStatus, &tsync);
+		RXAFS_Link(rxconn, (struct AFSFid *)&adp->f.fid.Fid, aname,
+			   (struct AFSFid *)&avc->f.fid.Fid, OutFidStatus,
+			   OutDirStatus, &tsync);
 	    RX_AFS_GLOCK();
 	    XSTATS_END_TIME;
 
 	} else
 	    code = -1;
     } while (afs_Analyze
-	     (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_LINK,
+	     (tc, rxconn, code, &adp->f.fid, treq, AFS_STATS_FS_RPCIDX_LINK,
 	      SHARED_LOCK, NULL));
 
     if (code) {
 	if (tdc)
 	    afs_PutDCache(tdc);
 	if (code < 0) {
-	    ObtainWriteLock(&afs_xcbhash, 492);
-	    afs_DequeueCallback(adp);
-	    adp->f.states &= ~CStatd;
-	    ReleaseWriteLock(&afs_xcbhash);
-	    osi_dnlc_purgedp(adp);
+	    afs_StaleVCache(adp);
 	}
 	ReleaseWriteLock(&adp->lock);
 	goto done;
     }
     if (tdc)
 	ObtainWriteLock(&tdc->lock, 635);
-    if (afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
+    if (afs_LocalHero(adp, tdc, OutDirStatus, 1)) {
 	/* we can do it locally */
 	ObtainWriteLock(&afs_xdcache, 290);
 	code = afs_dir_Create(tdc, aname, &avc->f.fid.Fid);
@@ -153,19 +152,17 @@ afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname,
      * returned in ustat is the most recent to store in the file's
      * cache entry */
 
-    ObtainWriteLock(&afs_xcbhash, 493);
-    afs_DequeueCallback(avc);
-    avc->f.states &= ~CStatd;	/* don't really know new link count */
-    ReleaseWriteLock(&afs_xcbhash);
-    if (avc->f.fid.Fid.Vnode & 1 || (vType(avc) == VDIR))
-	osi_dnlc_purgedp(avc);
+    afs_StaleVCache(avc); /* don't really know new link count */
     ReleaseWriteLock(&avc->lock);
     code = 0;
   done:
-    code = afs_CheckCode(code, &treq, 24);
+    code = afs_CheckCode(code, treq, 24);
+    afs_DestroyReq(treq);
     afs_PutFakeStat(&vfakestate);
     afs_PutFakeStat(&dfakestate);
     AFS_DISCON_UNLOCK();
   done2:
+    osi_FreeSmallSpace(OutFidStatus);
+    osi_FreeSmallSpace(OutDirStatus);
     return code;
 }
