@@ -66,7 +66,7 @@ static struct ltable {
 };
 unsigned long lastCallBack_vnode;
 unsigned int lastCallBack_dv;
-osi_timeval_t lastCallBack_time;
+osi_timeval32_t lastCallBack_time;
 
 /* these are for storing alternate interface addresses */
 struct interfaceAddr afs_cb_interface;
@@ -447,9 +447,6 @@ loop1:
 			    afs_osi_Sleep(&tvc->f.states);
 			    goto loop1;
 			}
-#if     defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
-			AFS_FAST_HOLD(tvc);
-#else
 #ifdef AFS_DARWIN80_ENV
 			if (tvc->f.states & CDeadVnode) {
 			    ReleaseReadLock(&afs_xvcache);
@@ -466,8 +463,9 @@ loop1:
 			    continue;
 			}
 #else
-			AFS_FAST_HOLD(tvc);
-#endif
+			if (osi_vnhold(tvc) != 0) {
+			    continue;
+			}
 #endif
 			ReleaseReadLock(&afs_xvcache);
 			afs_StaleVCacheFlags(tvc, 0, CUnique | CBulkFetching);
@@ -530,9 +528,6 @@ loop2:
 			afs_osi_Sleep(&tvc->f.states);
 			goto loop2;
 		    }
-#if     defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)  || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV)
-		    AFS_FAST_HOLD(tvc);
-#else
 #ifdef AFS_DARWIN80_ENV
 		    if (tvc->f.states & CDeadVnode) {
 			ReleaseReadLock(&afs_xvcache);
@@ -549,8 +544,9 @@ loop2:
 			continue;
 		    }
 #else
-		    AFS_FAST_HOLD(tvc);
-#endif
+		    if (osi_vnhold(tvc) != 0) {
+			continue;
+		    }
 #endif
 		    ReleaseReadLock(&afs_xvcache);
 		    afs_StaleVCacheFlags(tvc, 0, CUnique | CBulkFetching);
@@ -560,7 +556,7 @@ loop2:
 #ifdef CBDEBUG
 		    lastCallBack_vnode = afid->Vnode;
 		    lastCallBack_dv = tvc->mstat.DataVersion.low;
-		    osi_GetuTime(&lastCallBack_time);
+		    osi_GetTime(&lastCallBack_time);
 #endif /* CBDEBUG */
 #ifdef AFS_DARWIN80_ENV
 		    vnode_put(AFSTOV(tvc));
@@ -1637,14 +1633,3 @@ SRXAFSCB_TellMeAboutYourself(struct rx_call *a_call,
 
     return code;
 }
-
-
-int
-SRXAFSCB_GetDE(struct rx_call *a_call, afs_int32 a_index, afs_int32 *addr,
-	       afs_int32 *inode, afs_int32 *flags, afs_int32 *time,
-	       char ** fileName)
-{ /*SRXAFSCB_GetDE*/
-    int code = 0;				/*Return code*/
-    return(code);
-
-} /*SRXAFSCB_GetDE*/
