@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,24 +9,13 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <fcntl.h>
-#include <sys/stat.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#include <WINNT/afsevent.h>
-#else
-#include <netinet/in.h>
-#include <sys/file.h>
-#include <sys/time.h>
-#include <netdb.h>
-#endif
-#include <string.h>
 #include <afs/stds.h>
-#include <sys/types.h>
-#include <time.h>
-#include <stdio.h>
+
+#include <roken.h>
+
+#ifdef AFS_NT40_ENV
+#include <WINNT/afsevent.h>
+#endif
 #include <afs/cmd.h>
 #include <lwp.h>
 #include <ubik.h>
@@ -35,19 +24,17 @@
 #include <rx/rxkad.h>
 #include <rx/rx_globals.h>
 #include <afs/cellconfig.h>
+#include <afs/authcon.h>
 #include <afs/bubasics.h>
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
-#include <errno.h>
-#ifdef	AFS_AIX32_ENV
-#include <signal.h>
-#endif
+#include <afs/audit.h>
+
 #include "budb_errs.h"
 #include "database.h"
 #include "error_macros.h"
 #include "budb_internal.h"
 #include "globals.h"
-#include "afs/audit.h"
 
 struct ubik_dbase *BU_dbase;
 struct afsconf_dir *BU_conf;	/* for getting cell info */
@@ -57,8 +44,9 @@ int truncateDatabase(void);
 int parseServerList(struct cmd_item *);
 
 char lcell[MAXKTCREALMLEN];
-afs_int32 myHost = 0;
+afs_uint32 myHost = 0;
 int helpOption;
+static struct logOptions logopts;
 
 /* server's global configuration information. This is exported to other
  * files/routines
@@ -79,14 +67,6 @@ int lwps   = 3;
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
 
-#if defined(AFS_PTHREAD_ENV)
-static int
-threadNum(void)
-{
-    return (intptr_t)pthread_getspecific(rx_thread_id_key);
-}
-#endif
-
 /* check whether caller is authorized to manage RX statistics */
 int
 BU_rxstat_userok(struct rx_call *call)
@@ -94,9 +74,27 @@ BU_rxstat_userok(struct rx_call *call)
     return afsconf_SuperUser(BU_conf, call, NULL);
 }
 
+/**
+ * Return true if this name is a member of the local realm.
+ */
 int
-convert_cell_to_ubik(struct afsconf_cell *cellinfo, afs_int32 *myHost, 
-		     afs_int32 *serverList)
+BU_IsLocalRealmMatch(void *rock, char *name, char *inst, char *cell)
+{
+    struct afsconf_dir *dir = (struct afsconf_dir *)rock;
+    afs_int32 islocal = 0;	/* default to no */
+    int code;
+
+    code = afsconf_IsLocalRealmMatch(dir, &islocal, name, inst, cell);
+    if (code) {
+	LogError(code, "Failed local realm check; name=%s, inst=%s, cell=%s\n",
+		 name, inst, cell);
+    }
+    return islocal;
+}
+
+int
+convert_cell_to_ubik(struct afsconf_cell *cellinfo, afs_uint32 *myHost,
+		     afs_uint32 *serverList)
 {
     int i;
     char hostname[64];
@@ -109,7 +107,7 @@ convert_cell_to_ubik(struct afsconf_cell *cellinfo, afs_int32 *myHost,
 	printf("prserver: couldn't get address of this host.\n");
 	BUDB_EXIT(1);
     }
-    memcpy(myHost, th->h_addr, sizeof(afs_int32));
+    memcpy(myHost, th->h_addr, sizeof(afs_uint32));
 
     for (i = 0; i < cellinfo->numServers; i++)
 	/* omit my host from serverList */
@@ -126,7 +124,7 @@ convert_cell_to_ubik(struct afsconf_cell *cellinfo, afs_int32 *myHost,
  *      If it were, this routine would never have been called.
  */
 static int
-MyBeforeProc(register struct cmd_syndesc *as, void *arock)
+MyBeforeProc(struct cmd_syndesc *as, void *arock)
 {
     helpOption = 0;
     return 0;
@@ -143,7 +141,7 @@ initializeArgHandler(void)
 
     cmd_SetBeforeProc(MyBeforeProc, NULL);
 
-    cptr = cmd_CreateSyntax(NULL, argHandler, NULL, "Backup database server");
+    cptr = cmd_CreateSyntax(NULL, argHandler, NULL, 0, "Backup database server");
 
     cmd_AddParm(cptr, "-database", CMD_SINGLE, CMD_OPTIONAL,
 		"database directory");
@@ -166,8 +164,8 @@ initializeArgHandler(void)
     cmd_AddParm(cptr, "-ubikbuffers", CMD_SINGLE, CMD_OPTIONAL,
 		"the number of ubik buffers");
 
-    cmd_AddParm(cptr, "-auditlog", CMD_SINGLE, CMD_OPTIONAL,
-		"audit log path");
+    cmd_AddParm(cptr, "-auditlog", CMD_LIST, CMD_OPTIONAL,
+		"[interface:]path[:options]");
 
     cmd_AddParm(cptr, "-p", CMD_SINGLE, CMD_OPTIONAL,
 		"number of processes");
@@ -176,33 +174,33 @@ initializeArgHandler(void)
 		"bind the Rx socket (primary interface only)");
 
     cmd_AddParm(cptr, "-audit-interface", CMD_SINGLE, CMD_OPTIONAL,
-		"audit interface (file or sysvmq)");
+		"default interface");
+
+    cmd_AddParm(cptr, "-transarc-logs", CMD_FLAG, CMD_OPTIONAL,
+		"enable Transarc style logging");
 }
 
 int
 argHandler(struct cmd_syndesc *as, void *arock)
 {
 
+    char *auditIface = NULL;
+
     /* globalConfPtr provides the handle for the configuration information */
 
     /* database directory */
     if (as->parms[0].items != 0) {
-	globalConfPtr->databaseDirectory =
-	    (char *)malloc(strlen(as->parms[0].items->data) + 1);
+	globalConfPtr->databaseDirectory = strdup(as->parms[0].items->data);
 	if (globalConfPtr->databaseDirectory == 0)
 	    BUDB_EXIT(-1);
-	strcpy(globalConfPtr->databaseDirectory, as->parms[0].items->data);
     }
 
     /* -cellservdb, cell configuration directory */
     if (as->parms[1].items != 0) {
-	globalConfPtr->cellConfigdir =
-	    (char *)malloc(strlen(as->parms[1].items->data) + 1);
+	globalConfPtr->cellConfigdir = strdup(as->parms[1].items->data);
 	if (globalConfPtr->cellConfigdir == 0)
 	    BUDB_EXIT(-1);
 
-	strcpy(globalConfPtr->cellConfigdir, as->parms[1].items->data);
-
 	globalConfPtr->debugFlags |= DF_RECHECKNOAUTH;
     }
 
@@ -250,23 +248,21 @@ argHandler(struct cmd_syndesc *as, void *arock)
 	rxBind = 1;
     }
 
-    /* -audit-interface */
-    if (as->parms[10].items != 0) {
-	char *interface = as->parms[10].items->data;
+    /* -audit-interface and -auditlog */
+    if (as->parms[10].items != 0)
+	auditIface = as->parms[10].items->data;
 
-	if (osi_audit_interface(interface)) {
-	    printf("Invalid audit interface '%s'\n", interface);
+    if (as->parms[7].items != 0) {
+	int code;
+	code = osi_audit_cmd_Options(auditIface, as->parms[7].items);
+	if (code)
 	    BUDB_EXIT(-1);
-	}
     }
 
-    /* -auditlog */
-    /* needs to be after -audit-interface, so we osi_audit_interface
-     * before we osi_audit_file */
-    if (as->parms[7].items != 0) {
-	char *fileName = as->parms[7].items->data;
-
-        osi_audit_file(fileName);
+    /* -transarc-logs */
+    if (as->parms[11].items != 0) {
+	logopts.lopt_rotateOnOpen = 1;
+	logopts.lopt_rotateStyle = logRotate_old;
     }
 
     return 0;
@@ -294,7 +290,7 @@ parseServerList(struct cmd_item *itemPtr)
     LogDebug(3, "%d servers\n", nservers);
 
     /* now can allocate the space for the server arguments */
-    serverArgs = (char **)malloc((nservers + 2) * sizeof(char *));
+    serverArgs = malloc((nservers + 2) * sizeof(char *));
     if (serverArgs == 0)
 	ERROR(-1);
 
@@ -316,7 +312,7 @@ parseServerList(struct cmd_item *itemPtr)
 	ERROR(code);
 
     /* free space for the server args */
-    free((char *)serverArgs);
+    free(serverArgs);
 
   error_exit:
     return (code);
@@ -331,20 +327,15 @@ truncateDatabase(void)
 {
     char *path;
     afs_int32 code = 0;
-    int fd;
+    int fd,r;
 
-    path =
-	(char *)malloc(strlen(globalConfPtr->databaseDirectory) +
-		       strlen(globalConfPtr->databaseName) +
-		       strlen(globalConfPtr->databaseExtension) + 1);
-    if (path == 0)
+    r = asprintf(&path, "%s%s%s",
+		 globalConfPtr->databaseDirectory,
+		 globalConfPtr->databaseName,
+		 globalConfPtr->databaseExtension);
+    if (r < 0 || path == NULL)
 	ERROR(-1);
 
-    /* construct the database name */
-    strcpy(path, globalConfPtr->databaseDirectory);
-    strcat(path, globalConfPtr->databaseName);
-    strcat(path, globalConfPtr->databaseExtension);
-
     fd = open(path, O_RDWR, 0755);
     if (!fd) {
 	code = errno;
@@ -355,6 +346,8 @@ truncateDatabase(void)
 	    close(fd);
     }
 
+    free(path);
+
   error_exit:
     return (code);
 }
@@ -369,16 +362,20 @@ main(int argc, char **argv)
 {
     char *whoami = argv[0];
     char *dbNamePtr = 0;
-    struct afsconf_cell cellinfo;
+    struct afsconf_cell cellinfo_s;
+    struct afsconf_cell *cellinfo = NULL;
     time_t currentTime;
     afs_int32 code = 0;
+    char hoststr[16];
     afs_uint32 host = ntohl(INADDR_ANY);
+    int r;
 
     char  clones[MAXHOSTSPERCELL];
 
     struct rx_service *tservice;
     struct rx_securityClass **securityClasses;
     afs_int32 numClasses;
+    struct afsconf_bsso_info bsso;
 
     extern int rx_stackSize;
 
@@ -393,8 +390,8 @@ main(int argc, char **argv)
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -406,8 +403,16 @@ main(int argc, char **argv)
     sigaction(SIGSEGV, &nsa, NULL);
     sigaction(SIGABRT, &nsa, NULL);
 #endif
+
+    memset(&cellinfo_s, 0, sizeof(cellinfo_s));
+    memset(clones, 0, sizeof(clones));
+    memset(&bsso, 0, sizeof(bsso));
+
+    memset(&logopts, 0, sizeof(logopts));
+    logopts.lopt_dest = logDest_file;
+    logopts.lopt_filename = AFSDIR_SERVER_BUDBLOG_FILEPATH;
+
     osi_audit_init();
-    osi_audit(BUDB_StartEvent, 0, AUD_END);
 
     initialize_BUDB_error_table();
     initializeArgHandler();
@@ -431,20 +436,10 @@ main(int argc, char **argv)
     strcpy(cellConfDir, AFSDIR_SERVER_ETC_DIRPATH);
     globalConfPtr->cellConfigdir = cellConfDir;
 
-    /* open the log file */
-/*
-    globalConfPtr->log = fopen(DEFAULT_LOGNAME,"a");
-    if ( globalConfPtr->log == NULL )
-    {   
-	printf("Can't open log file %s - aborting\n", DEFAULT_LOGNAME);
-	BUDB_EXIT(-1);
-    }
-*/
-
     srandom(1);
 
 #ifdef AFS_PTHREAD_ENV
-    SetLogThreadNumProgram( threadNum );
+    SetLogThreadNumProgram( rx_GetThreadNum );
 #endif
 
     /* process the user supplied args */
@@ -457,17 +452,12 @@ main(int argc, char **argv)
     if (helpOption)
 	BUDB_EXIT(0);
 
+    /* Start auditing */
+    osi_audit_open();
+    osi_audit(BUDB_StartEvent, 0, AUD_END);
+
     /* open the log file */
-    globalConfPtr->log = fopen(AFSDIR_SERVER_BUDBLOG_FILEPATH, "a");
-    if (globalConfPtr->log == NULL) {
-	printf("Can't open log file %s - aborting\n",
-	       AFSDIR_SERVER_BUDBLOG_FILEPATH);
-	BUDB_EXIT(-1);
-    }
-
-    /* keep log closed so can remove it */
-
-    fclose(globalConfPtr->log);
+    OpenLog(&logopts);
 
     /* open the cell's configuration directory */
     LogDebug(4, "opening %s\n", globalConfPtr->cellConfigdir);
@@ -490,69 +480,87 @@ main(int argc, char **argv)
 	 * list from the cell's database
 	 */
 
+	cellinfo = &cellinfo_s;
+
 	LogDebug(1, "Using server list from %s cell database.\n", lcell);
 
-	code = afsconf_GetExtendedCellInfo (BU_conf, lcell, 0, &cellinfo,
+	code = afsconf_GetExtendedCellInfo (BU_conf, lcell, 0, cellinfo,
 					    clones);
+	if (code) {
+	    LogError(0, "Can't read cell information\n");
+	    ERROR(code);
+	}
+
 	code =
-	    convert_cell_to_ubik(&cellinfo, &globalConfPtr->myHost,
+	    convert_cell_to_ubik(cellinfo, &globalConfPtr->myHost,
 				 globalConfPtr->serverList);
 	if (code)
 	    ERROR(code);
     }
 
+    /* initialize audit user check */
+    osi_audit_set_user_check(BU_conf, BU_IsLocalRealmMatch);
+
     /* initialize ubik */
-    ubik_CRXSecurityProc = afsconf_ClientAuth;
-    ubik_CRXSecurityRock = BU_conf;
-
-    ubik_SRXSecurityProc = afsconf_ServerAuth;
-    ubik_SRXSecurityRock = BU_conf;
-
-    ubik_CheckRXSecurityProc = afsconf_CheckAuth;
-    ubik_CheckRXSecurityRock = BU_conf;
+    ubik_SetClientSecurityProcs(afsconf_ClientAuth, afsconf_UpToDate, BU_conf);
+    ubik_SetServerSecurityProcs(afsconf_BuildServerSecurityObjects,
+				afsconf_CheckAuth, BU_conf);
 
     if (ubik_nBuffers == 0)
 	ubik_nBuffers = 400;
 
     LogError(0, "Will allocate %d ubik buffers\n", ubik_nBuffers);
 
-    dbNamePtr =
-	(char *)malloc(strlen(globalConfPtr->databaseDirectory) +
-		       strlen(globalConfPtr->databaseName) + 1);
-    if (dbNamePtr == 0)
+    r = asprintf(&dbNamePtr, "%s%s", globalConfPtr->databaseDirectory,
+		 globalConfPtr->databaseName);
+    if (r < 0 || dbNamePtr == 0)
 	ERROR(-1);
 
-    /* construct the database name */
-    strcpy(dbNamePtr, globalConfPtr->databaseDirectory);
-    strcat(dbNamePtr, globalConfPtr->databaseName);	/* name prefix */
-
     rx_SetRxDeadTime(60);	/* 60 seconds inactive before timeout */
 
     if (rxBind) {
 	afs_int32 ccode;
-        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
+        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
-            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-                                           ADDRSPERSITE, reason,
-                                           AFSDIR_SERVER_NETINFO_FILEPATH,
-                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-        } else 
+            ccode = afsconf_ParseNetFiles(SHostAddrs, NULL, NULL,
+                                          ADDRSPERSITE, reason,
+                                          AFSDIR_SERVER_NETINFO_FILEPATH,
+                                          AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+        } else
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
         if (ccode == 1) {
             host = SHostAddrs[0];
-	    rx_InitHost(host, htons(AFSCONF_BUDBPORT));
 	}
     }
 
-    code = ubik_ServerInitByInfo (globalConfPtr->myHost,
-				  htons(AFSCONF_BUDBPORT), 
-				  &cellinfo,
-				  clones,              
-				  dbNamePtr,           /* name prefix */
-				  &BU_dbase);
+    Log("buserver binding rx to %s:%d\n",
+        afs_inet_ntoa_r(host, hoststr), AFSCONF_BUDBPORT);
+    code = rx_InitHost(host, htons(AFSCONF_BUDBPORT));
+    if (code) {
+	LogError(code, "rx init failed\n");
+	afs_com_err(whoami, code, "rx init failed");
+	ERROR(code);
+    }
+
+    /* Disable jumbograms */
+    rx_SetNoJumbo();
+
+    if (cellinfo) {
+	code = ubik_ServerInitByInfo(globalConfPtr->myHost,
+	                             htons(AFSCONF_BUDBPORT),
+	                             cellinfo,
+	                             clones,
+	                             dbNamePtr,           /* name prefix */
+	                             &BU_dbase);
+    } else {
+	code = ubik_ServerInit(globalConfPtr->myHost, htons(AFSCONF_BUDBPORT),
+	                       globalConfPtr->serverList,
+	                       dbNamePtr,  /* name prefix */
+	                       &BU_dbase);
+    }
 
     if (code) {
 	LogError(code, "Ubik init failed\n");
@@ -560,11 +568,10 @@ main(int argc, char **argv)
 	ERROR(code);
     }
 
-    afsconf_BuildServerSecurityObjects(BU_conf, 0,
-				       &securityClasses, &numClasses);
-
-    /* Disable jumbograms */
-    rx_SetNoJumbo();
+    bsso.dir = BU_conf;
+    bsso.logger = FSLog;
+    afsconf_BuildServerSecurityObjects_int(&bsso, &securityClasses,
+					   &numClasses);
 
     tservice =
 	rx_NewServiceHost(host, 0, BUDB_SERVICE, "BackupDatabase",
@@ -623,49 +630,21 @@ consistencyCheckDb(void)
 void
 LogDebug(int level, char *fmt, ... )
 {
-    va_list ap;
-
-    va_start(ap, fmt);
-	
     if (debugging >= level) {
-	/* log normally closed so can remove it */
-	globalConfPtr->log = fopen(AFSDIR_SERVER_BUDBLOG_FILEPATH, "a");
-	if (globalConfPtr->log != NULL) {
-	    vfprintf(globalConfPtr->log, fmt, ap);
-	    fflush(globalConfPtr->log);
-	    fclose(globalConfPtr->log);
-	}
+	va_list ap;
+	va_start(ap, fmt);
+	vFSLog(fmt, ap);
+	va_end(ap);
     }
-    va_end(ap);
-}
-
-static char *
-TimeStamp(time_t t)
-{
-    struct tm *lt;
-    static char timestamp[20];
-
-    lt = localtime(&t);
-    strftime(timestamp, 20, "%m/%d/%Y %H:%M:%S", lt);
-    return timestamp;
 }
 
 void
 Log(char *fmt, ...)
 {
     va_list ap;
-    time_t now;
 
     va_start(ap, fmt);
-    globalConfPtr->log = fopen(AFSDIR_SERVER_BUDBLOG_FILEPATH, "a");
-    if (globalConfPtr->log != NULL) {
-	now = time(0);
-	fprintf(globalConfPtr->log, "%s ", TimeStamp(now));
-
-	vfprintf(globalConfPtr->log, fmt, ap);
-	fflush(globalConfPtr->log);
-	fclose(globalConfPtr->log);
-    }
+    vFSLog(fmt, ap);
     va_end(ap);
 }
 
@@ -673,42 +652,21 @@ void
 LogError(long code, char *fmt, ... )
 {
     va_list ap;
-    time_t now;
+    int len;
+    char buffer[1024];
 
     va_start(ap, fmt);
-    globalConfPtr->log = fopen(AFSDIR_SERVER_BUDBLOG_FILEPATH, "a");
-
-    if (globalConfPtr->log != NULL) {
-	now = time(0);
-	fprintf(globalConfPtr->log, "%s ", TimeStamp(now));
-
-	if (code)
-	    fprintf(globalConfPtr->log, "%s: %s\n", afs_error_table_name(code),
-		    afs_error_message(code));
-	vfprintf(globalConfPtr->log, fmt, ap );
-	fflush(globalConfPtr->log);
-	fclose(globalConfPtr->log);
+    len = vsnprintf(buffer, sizeof(buffer), fmt, ap);
+    va_end(ap);
+    if (len >= 1024) {
+	len = 1023;
+	buffer[1023] = '\0';
+    }
+    /* Be consistent with (unintentional?) historic behavior. */
+    if (code) {
+	FSLog("%s: %s\n", afs_error_table_name(code), afs_error_message(code));
+	WriteLogBuffer(buffer, len);
+    } else {
+	FSLog("%s", buffer);
     }
 }
-
-
-/*  ----------------
- * debug
- * ----------------
- */
-
-void
-LogNetDump(struct dump *dumpPtr)
-{
-    struct dump hostDump;
-    extern buServerConfP globalConfPtr;
-
-    dump_ntoh(dumpPtr, &hostDump);
-
-    globalConfPtr->log = fopen(AFSDIR_SERVER_BUDBLOG_FILEPATH, "a");
-    if (globalConfPtr->log != NULL) {
-	printDump(globalConfPtr->log, &hostDump);
-	fclose(globalConfPtr->log);
-    }
-}
-
