@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,7 +14,7 @@
 #define _ICL_H__ENV_ 1
 
 #ifdef	KERNEL
-#include "param.h"
+#include "afs/param.h"
 #include "afs_osi.h"
 #include "lock.h"
 #include "afs_trace.h"
@@ -52,7 +52,7 @@ struct afs_icl_set {
 #define	ICL_SETF_PERSISTENT	8
 
 #ifdef ICL_DEFAULT_ENABLED
-#define ICL_DEFAULT_SET_STATES	0	/* was ICL_SETF_ACTIVE*/
+#define ICL_DEFAULT_SET_STATES	0	/* was ICL_SETF_ACTIVE */
 #else /* ICL_DEFAULT_ENABLED */
 
 #ifdef ICL_DEFAULT_DISABLED
@@ -66,13 +66,9 @@ struct afs_icl_set {
 #define ICL_DEFAULT_SET_STATES	0	/* was ICL_SETF_ACTIVE */
 #endif /* ICL_DEFAULT_SET_STATES */
 
-/* bytes required by eventFlags array, for x events */
-#define ICL_EVENTBYTES(x)	((((x) - 1) | 7) + 1)
-
 /* functions for finding a particular event */
 #define ICL_EVENTBYTE(x)	(((x) & 0x3ff) >> 3)
 #define ICL_EVENTMASK(x)	(1 << ((x) & 0x7))
-#define ICL_EVENTOK(setp, x)	((x&0x3ff) >= 0 && (x&0x3ff) < (setp)->nevents)
 
 /* define ICL syscalls by name!! */
 #define ICL_OP_COPYOUT		1
@@ -96,9 +92,6 @@ struct afs_icl_set {
 #define ICL_OP_SS_DEACTIVATE	2
 #define ICL_OP_SS_FREE		3
 
-/* define set status flags */
-#define	ICL_FLAG_ACTIVE		1
-#define	ICL_FLAG_FREED		2
 
 /* The format of the circular log is:
  * 1'st word:
@@ -129,13 +122,13 @@ struct afs_icl_log {
     char *name;			/* log name */
     struct afs_icl_log *nextp;	/* next log in system */
     afs_int32 logSize;		/* allocated # of elements in log */
-    afs_int32 logElements;		/* # of elements in log right now */
+    afs_int32 logElements;	/* # of elements in log right now */
     afs_int32 *datap;		/* pointer to the data */
-    afs_int32 firstUsed;		/* first element used */
-    afs_int32 firstFree;		/* index of first free dude */
-    long baseCookie;	/* cookie value of first entry */
+    afs_int32 firstUsed;	/* first element used */
+    afs_int32 firstFree;	/* index of first free dude */
+    long baseCookie;		/* cookie value of first entry */
     afs_int32 states;		/* state bits */
-    afs_uint32 lastTS;	/* last timestamp written to this log */
+    afs_uint32 lastTS;		/* last timestamp written to this log */
 };
 
 /* macro used to compute size of parameter when in log, used by
@@ -150,20 +143,20 @@ struct afs_icl_log {
  */
 extern int afs_icl_sizeofLong;
 
-#define ICL_SIZEHACK(t1, p1) \
+#define ICL_SIZEHACK(t1, p1, ts1, rs1)		\
     MACRO_BEGIN \
 	if ((t1) == ICL_TYPE_STRING) { \
-	    tsize = (int)((unsigned)(strlen((char *)(p1)) + 4) >> 2); \
+	    ts1 = (int)((unsigned)(strlen((char *)(p1)) + 4) >> 2); \
 	} else if ((t1) == ICL_TYPE_HYPER  || (t1) == ICL_TYPE_INT64) \
-	    tsize = 2; \
+	    ts1 = 2; \
 	else if ((t1) == ICL_TYPE_FID) \
-	    tsize = 4; \
+	    ts1 = 4; \
 	else if ((t1) == ICL_TYPE_INT32) \
-	    tsize = 1; \
+	    ts1 = 1; \
 	else \
-	    tsize = afs_icl_sizeofLong; \
+	    ts1 = afs_icl_sizeofLong; \
 	/* now add in the parameter */ \
-	rsize += tsize; \
+	rs1 += ts1; \
     MACRO_END
 
 /* log flags */
@@ -230,7 +223,7 @@ extern int afs_icl_sizeofLong;
 #define lock_ReleaseWrite	ReleaseWriteLock
 
 #ifdef	KERNEL
-extern struct afs_icl_set *afs_iclSetp; /* standard icl trace */
+extern struct afs_icl_set *afs_iclSetp;	/* standard icl trace */
 /* A separate icl set to collect long term debugging info. */
 extern struct afs_icl_set *afs_iclLongTermSetp;
 #else
@@ -239,29 +232,8 @@ extern struct afs_icl_set *afs_iclLongTermSetp;
 
 #define ICL_RPC_MAX_SETS (64)
 #define ICL_RPC_MAX_LOGS (64)
- 
-typedef struct afs_icl_setinfo {
-    u_char setName[32];
-    afs_uint32 states;
-} afs_icl_setinfo_t;
- 
-typedef struct afs_icl_loginfo {
-    u_char logName[32];
-    afs_uint32 logSize;
-    afs_uint32 logElements;
-    afs_uint32 states;
-} afs_icl_loginfo_t;
- 
-typedef struct afs_icl_bulkSetinfo {
-    afs_uint32 count;
-    afs_icl_setinfo_t setinfo[1];
-} afs_icl_bulkSetinfo_t;
- 
-typedef struct afs_icl_bulkLoginfo {
-    afs_uint32 count;
-    afs_icl_loginfo_t loginfo[1];
-} afs_icl_bulkLoginfo_t;
- 
+
+
 #endif
 
 
