@@ -12,6 +12,7 @@
 
 #include <roken.h>
 
+#include <afs/opr.h>
 #include <lock.h>
 #include <ubik.h>
 
@@ -100,12 +101,10 @@ pr_WriteEntry(struct ubik_trans *tt, afs_int32 afd, afs_int32 pos, struct prentr
 	nentry.sibling = htonl(tentry->sibling);
 	nentry.child = htonl(tentry->child);
 	strncpy(nentry.name, tentry->name, PR_MAXNAMELEN);
-#ifdef PR_REMEMBER_TIMES
 	nentry.createTime = htonl(tentry->createTime);
 	nentry.addTime = htonl(tentry->addTime);
 	nentry.removeTime = htonl(tentry->removeTime);
 	nentry.changeTime = htonl(tentry->changeTime);
-#endif
 	for (i = 0; i < PRSIZE; i++)
 	    nentry.entries[i] = htonl(tentry->entries[i]);
 	tentry = &nentry;
@@ -149,12 +148,10 @@ pr_ReadEntry(struct ubik_trans *tt, afs_int32 afd, afs_int32 pos, struct prentry
     tentry->sibling = ntohl(nentry.sibling);
     tentry->child = ntohl(nentry.child);
     strncpy(tentry->name, nentry.name, PR_MAXNAMELEN);
-#ifdef PR_REMEMBER_TIMES
     tentry->createTime = ntohl(nentry.createTime);
     tentry->addTime = ntohl(nentry.addTime);
     tentry->removeTime = ntohl(nentry.removeTime);
     tentry->changeTime = ntohl(nentry.changeTime);
-#endif
     for (i = 0; i < PRSIZE; i++)
 	tentry->entries[i] = ntohl(nentry.entries[i]);
     return (code);
@@ -285,7 +282,7 @@ FindByID(struct ubik_trans *at, afs_int32 aid)
 	return 0;
     if (aid == tentry.id)
 	return entry;
-    osi_Assert(entry != tentry.nextID);
+    opr_Assert(entry != tentry.nextID);
     entry = tentry.nextID;
     while (entry != 0) {
 	memset(&tentry, 0, sizeof(tentry));
@@ -294,7 +291,7 @@ FindByID(struct ubik_trans *at, afs_int32 aid)
 	    return 0;
 	if (aid == tentry.id)
 	    return entry;
-	osi_Assert(entry != tentry.nextID);
+	opr_Assert(entry != tentry.nextID);
 	entry = tentry.nextID;
     }
     return 0;
@@ -318,7 +315,7 @@ FindByName(struct ubik_trans *at, char aname[PR_MAXNAMELEN], struct prentry *ten
 	return 0;
     if ((strncmp(aname, tentryp->name, PR_MAXNAMELEN)) == 0)
 	return entry;
-    osi_Assert(entry != tentryp->nextName);
+    opr_Assert(entry != tentryp->nextName);
     entry = tentryp->nextName;
     while (entry != 0) {
 	memset(tentryp, 0, sizeof(struct prentry));
@@ -327,7 +324,7 @@ FindByName(struct ubik_trans *at, char aname[PR_MAXNAMELEN], struct prentry *ten
 	    return 0;
 	if ((strncmp(aname, tentryp->name, PR_MAXNAMELEN)) == 0)
 	    return entry;
-	osi_Assert(entry != tentryp->nextName);
+	opr_Assert(entry != tentryp->nextName);
 	entry = tentryp->nextName;
     }
     return 0;
@@ -343,7 +340,8 @@ AllocID(struct ubik_trans *at, afs_int32 flag, afs_int32 *aid)
 
     if (flag & PRGRP) {
 	*aid = ntohl(cheader.maxGroup);
-	while (code && i < maxcount) {
+	/* Check for PRBADID to avoid wrap-around. */
+	while (code && i < maxcount && *aid != PRBADID) {
 	    --(*aid);
 	    code = FindByID(at, *aid);
 	    i++;
@@ -375,7 +373,7 @@ AllocID(struct ubik_trans *at, afs_int32 flag, afs_int32 *aid)
 	return PRSUCCESS;
     } else {
 	*aid = ntohl(cheader.maxID);
-	while (code && i < maxcount) {
+	while (code && i < maxcount && *aid != 0x7fffffff) {
 	    ++(*aid);
 	    code = FindByID(at, *aid);
 	    i++;
@@ -392,36 +390,6 @@ AllocID(struct ubik_trans *at, afs_int32 flag, afs_int32 *aid)
     }
 }
 
-afs_int32
-IDToName(struct ubik_trans *at, afs_int32 aid, char aname[PR_MAXNAMELEN])
-{
-    afs_int32 temp;
-    struct prentry tentry;
-    afs_int32 code;
-
-    temp = FindByID(at, aid);
-    if (temp == 0)
-	return PRNOENT;
-    code = pr_Read(at, 0, temp, (char *)&tentry, sizeof(tentry));
-    if (code)
-	return code;
-    strncpy(aname, tentry.name, PR_MAXNAMELEN);
-    return PRSUCCESS;
-}
-
-afs_int32
-NameToID(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid)
-{
-    afs_int32 temp;
-    struct prentry tentry;
-
-    temp = FindByName(at, aname, &tentry);
-    if (!temp)
-	return PRNOENT;
-    *aid = tentry.id;
-    return PRSUCCESS;
-}
-
 int
 IDCmp(const void *a, const void *b)
 {
@@ -457,7 +425,7 @@ RemoveFromIDHash(struct ubik_trans *tt, afs_int32 aid, afs_int32 *loc)		/* ??? i
     if (code)
 	return PRDBFAIL;
     while (aid != tentry.id) {
-	osi_Assert(trail != current);
+	opr_Assert(trail != current);
 	trail = current;
 	current = tentry.nextID;
 	if (current == 0)
@@ -482,6 +450,8 @@ RemoveFromIDHash(struct ubik_trans *tt, afs_int32 aid, afs_int32 *loc)		/* ??? i
 	    return PRDBFAIL;
 	bentry.nextID = tentry.nextID;
 	code = pr_WriteEntry(tt, 0, trail, &bentry);
+	if (code)
+	    return PRDBFAIL;
     }
     *loc = current;
     return PRSUCCESS;
@@ -535,7 +505,7 @@ RemoveFromNameHash(struct ubik_trans *tt, char *aname, afs_int32 *loc)
     if (code)
 	return PRDBFAIL;
     while (strcmp(aname, tentry.name)) {
-	osi_Assert(trail != current);
+	opr_Assert(trail != current);
 	trail = current;
 	current = tentry.nextName;
 	if (current == 0)
@@ -560,6 +530,8 @@ RemoveFromNameHash(struct ubik_trans *tt, char *aname, afs_int32 *loc)
 	    return PRDBFAIL;
 	bentry.nextName = tentry.nextName;
 	code = pr_WriteEntry(tt, 0, trail, &bentry);
+	if (code)
+	    return PRDBFAIL;
     }
     *loc = current;
     return PRSUCCESS;
