@@ -12,15 +12,15 @@
 #include <afs/stds.h>
 
 #include <roken.h>
+#include <afs/opr.h>
+#ifdef AFS_PTHREAD_ENV
+# include <opr/lock.h>
+#endif
 
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wstrict-prototypes"
 #endif
 
-#ifdef UKERNEL
-#include "afsincludes.h"
-#endif
-
 #include <afs/pthread_glock.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
@@ -83,7 +83,6 @@ ubik_ParseClientList(int argc, char **argv, afs_uint32 * aothers)
     return 0;
 }
 
-#include <afs/afs_assert.h>
 #ifdef AFS_PTHREAD_ENV
 #include <pthread.h>
 
@@ -94,13 +93,12 @@ static pthread_key_t random_number_key;
 static void
 afs_random_once(void)
 {
-    osi_Assert(pthread_key_create(&random_number_key, NULL) == 0);
+    opr_Verify(pthread_key_create(&random_number_key, NULL) == 0);
     called_afs_random_once = 1;
 }
 
 #endif
 
-#if !defined(UKERNEL)
 /*!
  * \brief use time and pid to try to get some initial randomness.
  */
@@ -176,7 +174,6 @@ afs_randomMod15(void)
 
     return temp;
 }
-#endif /* !defined(UKERNEL) */
 
 #ifdef abs
 #undef abs
@@ -218,13 +215,14 @@ ubik_ClientInit(struct rx_connection **serverconns,
 	    return UMUTEXDESTROY;
 #endif
     } else {
-	tc = (struct ubik_client *)malloc(sizeof(struct ubik_client));
+	tc = malloc(sizeof(struct ubik_client));
     }
     if (tc == NULL)
 	return UNOMEM;
     memset((void *)tc, 0, sizeof(*tc));
 #ifdef AFS_PTHREAD_ENV
     if (pthread_mutex_init(&(tc->cm), (const pthread_mutexattr_t *)0)) {
+	free(tc);
 	return UMUTEXINIT;
     }
 #endif
@@ -323,7 +321,7 @@ ubik_RefreshConn(struct rx_connection *tc)
 pthread_once_t ubik_client_once = PTHREAD_ONCE_INIT;
 pthread_mutex_t ubik_client_mutex;
 #define LOCK_UCLNT_CACHE do { \
-    osi_Assert(pthread_once(&ubik_client_once, ubik_client_init_mutex) == 0); \
+    opr_Verify(pthread_once(&ubik_client_once, ubik_client_init_mutex) == 0); \
     MUTEX_ENTER(&ubik_client_mutex); \
   } while (0)
 #define UNLOCK_UCLNT_CACHE MUTEX_EXIT(&ubik_client_mutex)
@@ -347,62 +345,6 @@ static int synccount = 0;
 
 
 
-/*!
- * \brief Call this after getting back a #UNOTSYNC.
- *
- * \note Getting a #UNOTSYNC error code back does \b not guarantee
- * that there is a sync site yet elected.  However, if there is a sync
- * site out there somewhere, and you're trying an operation that
- * requires a sync site, ubik will return #UNOTSYNC, indicating the
- * operation won't work until you find a sync site
- */
-static int
-try_GetSyncSite(struct ubik_client *aclient, afs_int32 apos)
-{
-    struct rx_peer *rxp;
-    afs_int32 code;
-    int i;
-    afs_int32 thisHost, newHost;
-    struct rx_connection *tc;
-    short origLevel;
-
-    origLevel = aclient->initializationState;
-
-    /* get this conn */
-    tc = aclient->conns[apos];
-    if (tc && rx_ConnError(tc)) {
-	aclient->conns[apos] = (tc = ubik_RefreshConn(tc));
-    }
-    if (!tc) {
-	return -1;
-    }
-
-    /* now see if we can find the sync site host */
-    code = VOTE_GetSyncSite(tc, &newHost);
-    if (aclient->initializationState != origLevel) {
-	return -1;		/* somebody did a ubik_ClientInit */
-    }
-
-    if (!code && newHost) {
-	newHost = htonl(newHost);	/* convert back to network order */
-
-	/*
-	 * position count at the appropriate slot in the client
-	 * structure and retry. If we can't find in slot, we'll just
-	 * continue through the whole list
-	 */
-	for (i = 0; i < MAXSERVERS; i++) {
-	    rxp = rx_PeerOf(aclient->conns[i]);
-	    thisHost = rx_HostOf(rxp);
-	    if (!thisHost) {
-		return -1;
-	    } else if (thisHost == newHost) {
-		return i;	/* we were told to use this one */
-	    }
-	}
-    }
-    return -1;
-}
 
 #define NEED_LOCK 1
 #define NO_LOCK 0
@@ -432,12 +374,8 @@ CallIter(int (*aproc) (), struct ubik_client *aclient,
     while (*apos < MAXSERVERS) {
 	/* tc is the next conn to try */
 	tc = aclient->conns[*apos];
-	if (!tc) {
-	    if (needlock) {
-		UNLOCK_UBIK_CLIENT(aclient);
-	    }
-	    return UNOSERVERS;
-	}
+	if (!tc)
+	    goto errout;
 
 	if (rx_ConnError(tc)) {
 	    tc = ubik_RefreshConn(tc);
@@ -450,22 +388,15 @@ CallIter(int (*aproc) (), struct ubik_client *aclient,
 	    break;		/* this is the desired path */
 	}
     }
-    if (*apos >= MAXSERVERS) {
-	if (needlock) {
-	    UNLOCK_UBIK_CLIENT(aclient);
-	}
-	return UNOSERVERS;
-    }
+    if (*apos >= MAXSERVERS)
+	goto errout;
 
     code =
 	(*aproc) (tc, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13,
 		  p14, p15, p16);
-    if (aclient->initializationState != origLevel) {
-	if (needlock) {
-	    UNLOCK_UBIK_CLIENT(aclient);
-	}
-	return code;		/* somebody did a ubik_ClientInit */
-    }
+    if (aclient->initializationState != origLevel)
+	/* somebody did a ubik_ClientInit */
+	goto errout;
 
     /* what should I do in case of UNOQUORUM ? */
     if (code < 0) {
@@ -476,6 +407,7 @@ CallIter(int (*aproc) (), struct ubik_client *aclient,
     }
 
     (*apos)++;
+errout:
     if (needlock) {
 	UNLOCK_UBIK_CLIENT(aclient);
     }
@@ -509,9 +441,7 @@ ubik_Call_New(int (*aproc) (), struct ubik_client *aclient,
 {
     afs_int32 code, rcode;
     afs_int32 count;
-    afs_int32 temp;
     int pass;
-    int stepBack;
     short origLevel;
 
     LOCK_UBIK_CLIENT(aclient);
@@ -522,7 +452,6 @@ ubik_Call_New(int (*aproc) (), struct ubik_client *aclient,
     /* Do two passes. First pass only checks servers known running */
     for (aflags |= UPUBIKONLY, pass = 0; pass < 2;
 	 pass++, aflags &= ~UPUBIKONLY) {
-	stepBack = 0;
 	count = 0;
 	while (1) {
 	    code =
@@ -537,17 +466,7 @@ ubik_Call_New(int (*aproc) (), struct ubik_client *aclient,
 	    }
 	    rcode = code;	/* remember code from last good call */
 
-	    if (code == UNOTSYNC) {	/* means this requires a sync site */
-		if (aclient->conns[3]) {	/* don't bother unless 4 or more srv */
-		    temp = try_GetSyncSite(aclient, count);
-		    if (aclient->initializationState != origLevel) {
-			goto restart;	/* somebody did a ubik_ClientInit */
-		    }
-		    if ((temp >= 0) && ((temp > count) || (stepBack++ <= 2))) {
-			count = temp;	/* generally try to make progress */
-		    }
-		}
-	    } else if ((code >= 0) && (code != UNOQUORUM)) {
+	    if ((code >= 0) && (code != UNOQUORUM) && (code != UNOTSYNC)) {
 		UNLOCK_UBIK_CLIENT(aclient);
 		return code;	/* success or global error condition */
 	    }
@@ -557,6 +476,34 @@ ubik_Call_New(int (*aproc) (), struct ubik_client *aclient,
     return rcode;
 }
 
+/**
+ * Find index of connection to ahost.
+ *
+ * @param[in]     aclient  client structure
+ * @param[in]     ahost    host to be found (IP address in net-byte order)
+ *
+ * @return index on success; -1 otherwise.
+ */
+static int
+IndexOf(struct ubik_client *aclient, int ahost)
+{
+    int i, thisHost, index = -1;
+    struct rx_peer *rxp;
+
+    for (i = 0; i < MAXSERVERS && aclient->conns[i]; i++) {
+	rxp = rx_PeerOf(aclient->conns[i]);
+	thisHost = rx_HostOf(rxp);
+
+	if (!thisHost)
+	    break;
+	if (thisHost == ahost) {
+	    index = i;
+	    break;
+	}
+    }
+    return index;
+}
+
 /*!
  * call this instead of stub and we'll guarantee to find a host that's up.
  *
@@ -568,8 +515,8 @@ ubik_Call(int (*aproc) (), struct ubik_client *aclient,
 	  long p5, long p6, long p7, long p8, long p9, long p10,
 	  long p11, long p12, long p13, long p14, long p15, long p16)
 {
-    afs_int32 rcode, code, newHost, thisHost, i, count;
-    int chaseCount, pass, needsync, inlist, j;
+    afs_int32 rcode, code, count;
+    int pass, needsync, inlist, j;
     struct rx_connection *tc;
     struct rx_peer *rxp;
     short origLevel;
@@ -586,7 +533,7 @@ ubik_Call(int (*aproc) (), struct ubik_client *aclient,
   restart:
     origLevel = aclient->initializationState;
     rcode = UNOSERVERS;
-    chaseCount = inlist = needsync = 0;
+    inlist = needsync = 0;
 
     LOCK_UCLNT_CACHE;
     for (j = 0; ((j < SYNCCOUNT) && calls_needsync[j]); j++) {
@@ -607,46 +554,15 @@ ubik_Call(int (*aproc) (), struct ubik_client *aclient,
 	    if (needsync) {
 		/* Need a sync site. Lets try to quickly find it */
 		if (aclient->syncSite) {
-		    newHost = aclient->syncSite;	/* already in network order */
-		    aclient->syncSite = 0;	/* Will reset if it works */
-		} else if (aclient->conns[3]) {
-		    /* If there are fewer than four db servers in a cell,
-		     * there's no point in making the GetSyncSite call.
-		     * At best, it's a wash. At worst, it results in more
-		     * RPCs than you would otherwise make.
+		    /*
+		     * Position "count" at the appropriate slot in the client
+		     * structure and retry. If we can't find in slot, we'll just
+		     * continue through the whole list.
 		     */
-		    tc = aclient->conns[count];
-		    if (tc && rx_ConnError(tc)) {
-			aclient->conns[count] = tc = ubik_RefreshConn(tc);
-		    }
-		    if (!tc)
-			break;
-		    code = VOTE_GetSyncSite(tc, &newHost);
-		    if (aclient->initializationState != origLevel)
-			goto restart;	/* somebody did a ubik_ClientInit */
-		    if (code)
-			newHost = 0;
-		    newHost = htonl(newHost);	/* convert to network order */
-		} else {
-		    newHost = 0;
-		}
-		if (newHost) {
-		    /* position count at the appropriate slot in the client
-		     * structure and retry. If we can't find in slot, we'll
-		     * just continue through the whole list
-		     */
-		    for (i = 0; i < MAXSERVERS && aclient->conns[i]; i++) {
-			rxp = rx_PeerOf(aclient->conns[i]);
-			thisHost = rx_HostOf(rxp);
-			if (!thisHost)
-			    break;
-			if (thisHost == newHost) {
-			    if (chaseCount++ > 2)
-				break;	/* avoid loop asking */
-			    count = i;	/* this index is the sync site */
-			    break;
-			}
-		    }
+		    code = IndexOf(aclient, aclient->syncSite);
+		    aclient->syncSite = 0;
+		    if (code != -1)
+			count = code;
 		}
 	    }
 	    /*needsync */
@@ -700,3 +616,90 @@ ubik_Call(int (*aproc) (), struct ubik_client *aclient,
     UNLOCK_UBIK_CLIENT(aclient);
     return rcode;
 }
+
+afs_int32
+ubik_CallRock(struct ubik_client *aclient, afs_int32 aflags,
+	      ubik_callrock_func proc, void *rock)
+{
+    afs_int32 rcode, code, _ucount;
+    int pass, needsync;
+    struct rx_connection *tc;
+    struct rx_peer *rxp;
+    short origLevel;
+
+    if (!aclient)
+	return UNOENT;
+    LOCK_UBIK_CLIENT(aclient);
+
+ restart:
+    origLevel = aclient->initializationState;
+    rcode = UNOSERVERS;
+    needsync = 0;
+
+    /*
+     * First  pass, we try all servers that are up.
+     * Second pass, we try all servers.
+     */
+    for (pass = 0; pass < 2; pass++) {
+	/* For each entry in our servers list */
+	for (_ucount = 0;; _ucount++) {
+	    struct ubik_callrock_info info;
+	    if (needsync) {
+		/* Need a sync site. Lets try to quickly find it */
+		if (aclient->syncSite) {
+		    /*
+		     * Position "_ucount" at the appropriate slot in the client
+		     * structure and retry. If we can't find in slot, we'll just
+		     * continue through the whole list.
+		     */
+		    code = IndexOf(aclient, aclient->syncSite);
+		    aclient->syncSite = 0;
+		    if (code != -1)
+			_ucount = code;
+		}
+	    }
+	    /*needsync */
+	    tc = aclient->conns[_ucount];
+	    if (tc && rx_ConnError(tc)) {
+		aclient->conns[_ucount] = tc = ubik_RefreshConn(tc);
+	    }
+	    if (!tc)
+		break;
+
+	    if ((pass == 0) && (aclient->states[_ucount] & CFLastFailed)) {
+		continue;       /* this guy's down */
+	    }
+
+	    memset(&info, 0, sizeof(info));
+	    info.conn = tc;
+	    rcode = (*proc)(&info, rock);
+
+	    if (aclient->initializationState != origLevel) {
+		/* somebody did a ubik_ClientInit */
+		if (rcode)
+		    goto restart;       /* call failed */
+		else
+		    goto done;  /* call suceeded */
+	    }
+	    if (rcode < 0) {    /* network errors */
+		aclient->states[_ucount] |= CFLastFailed; /* Mark server down */
+	    } else if (rcode == UNOTSYNC) {
+		needsync = 1;
+	    } else if (rcode != UNOQUORUM) {
+		/* either misc ubik code, or misc appl code, or success. */
+		aclient->states[_ucount] &= ~CFLastFailed;	/* mark server up*/
+		goto done;      /* all done */
+	    }
+	}
+    }
+
+ done:
+    if (needsync) {
+	if (!rcode) {		/* Remember the sync site - cmd successful */
+	    rxp = rx_PeerOf(aclient->conns[_ucount]);
+	    aclient->syncSite = rx_HostOf(rxp);
+	}
+    }
+    UNLOCK_UBIK_CLIENT(aclient);
+    return rcode;
+}
