@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -12,23 +12,10 @@
  */
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <stdio.h>
-#include <sys/types.h>
-#include <unistd.h>
-#if !defined(AFS_SUN3_ENV) && !defined(sys_vax_ul43) 
-#include <time.h>
-/*#ifdef	AFS_AIX_ENV*/
-#include <sys/time.h>
-/*#endif*/
-#include <errno.h>
-#undef abs
-#include <stdlib.h>
-
-#include <string.h>
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
 #include <afs/cmd.h>
 #include <afs/afs_args.h>
 #include <afs/icl.h>
@@ -37,7 +24,7 @@
 #include <afs/vice.h>
 #include <afs/sys_prototypes.h>
 
-#if defined(AFS_OSF_ENV) || defined(AFS_SGI61_ENV) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_SGI61_ENV) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 /* For SGI 6.2, this is changed to 1 if it's a 32 bit kernel. */
 int afs_icl_sizeofLong = 2;
 #else
@@ -48,7 +35,6 @@ int afs_icl_sizeofLong = 1;
 int afs_64bit_kernel = 1;	/* Default for 6.2+, and always for 6.1 */
 extern int afs_icl_sizeofLong;	/* Used in ICL_SIZEHACK() */
 #ifdef AFS_SGI62_ENV
-#include <unistd.h>
 
 /* If _SC_KERN_POINTERS not in sysconf, then we can assume a 32 bit abi. */
 void
@@ -74,43 +60,16 @@ int afs_syscall(long call, long parm0, long parm1, long parm2, long parm3,
 		long parm4, long parm5, long parm6);
 void dce1_error_inq_text(afs_uint32 status_to_convert,
 		         char *error_text, int *status);
-int icl_CreateSetWithFlags(char *name, struct afs_icl_log *baseLogp,
-		           struct afs_icl_log *fatalLogp, afs_uint32 flags,
-		           struct afs_icl_set **outSetpp);
-int icl_LogHold(register struct afs_icl_log *logp);
-int icl_LogUse(register struct afs_icl_log *logp);
-int icl_LogReleNL(register struct afs_icl_log *logp);
-int icl_LogRele(register struct afs_icl_log *logp);
-int icl_ZeroLog(register struct afs_icl_log *logp);
-int icl_LogFreeUse(register struct afs_icl_log *logp);
 
 #define BUFFER_MULTIPLIER     1024
 
-/* make it big enough to snapshot everything at once, since
- * decoding takes so long.
- */
-#define IBSIZE		100000	/* default size */
-
 struct logInfo {
     struct logInfo *nextp;
     char *name;
 } *allInfo = 0;
 
-char dumpFileName[256] = "";
-void
-RegisterIclDumpFileName(char *name)
-{
-    (void)sprintf(dumpFileName, "icl.%.250s", name);
-}
-
-/* define globals to use for bulk info */
-afs_icl_bulkSetinfo_t *setInfo = (afs_icl_bulkSetinfo_t *) 0;
-afs_icl_bulkLoginfo_t *logInfo = (afs_icl_bulkLoginfo_t *) 0;
-
-struct afs_icl_set *icl_allSets = 0;
 
 
-char *name;
 /* given a type and an address, get the size of the thing
  * in words.
  */
@@ -121,7 +80,7 @@ icl_GetSize(afs_int32 type, char *addr)
     int tsize;
 
     rsize = 0;
-    ICL_SIZEHACK(type, addr);
+    ICL_SIZEHACK(type, addr, tsize, rsize);
     return rsize;
 }
 
@@ -134,7 +93,7 @@ icl_GetSize(afs_int32 type, char *addr)
 static int
 CheckTypes(char *bufferp, int *typesp, int typeCount, char *outMsgBuffer)
 {
-    register char tc;
+    char tc;
     int inPercent;
     int tix;
 
@@ -171,7 +130,7 @@ CheckTypes(char *bufferp, int *typesp, int typeCount, char *outMsgBuffer)
 	     * check the type descriptor.
 	     */
 	    inPercent = 0;
-	    if (tix > typeCount)
+	    if (tix >= typeCount)
 		return 0;	/* no more type descriptors left */
 	    if (tc == 's') {
 		if (typesp[tix] != 1)	/* not a string descriptor */
@@ -209,7 +168,7 @@ CheckTypes(char *bufferp, int *typesp, int typeCount, char *outMsgBuffer)
 static int
 CheckTypes(char *bufferp, int *typesp, int typeCount)
 {
-    register char tc;
+    char tc;
     int inPercent;
     int tix;
 
@@ -237,7 +196,7 @@ CheckTypes(char *bufferp, int *typesp, int typeCount)
 	     * check the type descriptor.
 	     */
 	    inPercent = 0;
-	    if (tix > typeCount)
+	    if (tix >= typeCount)
 		return 0;	/* no more type descriptors left */
 	    if (tc == 's' && typesp[tix] != 1)	/* not a string descriptor */
 		return 0;
@@ -260,7 +219,7 @@ CheckTypes(char *bufferp, int *typesp, int typeCount)
 #define uint64_t long long
 #endif
 static void
-DisplayRecord(FILE *outFilep, register afs_int32 *alp, afs_int32 rsize)
+DisplayRecord(FILE *outFilep, afs_int32 *alp, afs_int32 rsize)
 {
     char msgBuffer[1024];
 #if defined(AFS_SGI61_ENV) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
@@ -310,10 +269,7 @@ DisplayRecord(FILE *outFilep, register afs_int32 *alp, afs_int32 rsize)
 		printfParms[pfpix] <<= 32;
 		printfParms[pfpix] |= alp[pix + 1];
 	    }
-#elif defined(AFS_OSF_ENV)
-	    printfParms[pfpix] = alp[pix + 1];
-	    printfParms[pfpix] |= (alp[pix] <<= 32);
-#else /* !AFS_OSF_ENV && !AFS_SGI61_ENV */
+#else /* !AFS_SGI61_ENV */
 	    printfParms[pfpix] = alp[pix];
 #endif
 	    pfpix++;
@@ -524,9 +480,6 @@ DisplayRecord(FILE *outFilep, register afs_int32 *alp, afs_int32 rsize)
 
 
 #include <locale.h>
-#ifdef	AFS_OSF_ENV
-#include <limits.h>
-#endif
 #include <nl_types.h>
 
 #define FACILITY_CODE_MASK          0xF0000000
@@ -548,7 +501,7 @@ DisplayRecord(FILE *outFilep, register afs_int32 *alp, afs_int32 rsize)
  */
 
 void
-dce1_error_inq_text(afs_uint32 status_to_convert, 
+dce1_error_inq_text(afs_uint32 status_to_convert,
 		   char *error_text, int *status)
 {
     unsigned short facility_code;
@@ -622,8 +575,8 @@ dce1_error_inq_text(afs_uint32 status_to_convert,
      * install this special message catalog in with internationalization
      * catalogs.
      */
-    afs_snprintf(nls_filename, sizeof(nls_filename), "%s/C/%s.cat",
-		 AFSDIR_CLIENT_DATA_DIRPATH, filename_prefix);
+    snprintf(nls_filename, sizeof(nls_filename), "%s/C/%s.cat",
+	     AFSDIR_CLIENT_DATA_DIRPATH, filename_prefix);
 
     catd = catopen(nls_filename, 0);
     if (catd == (nl_catd) -1) {
@@ -675,7 +628,6 @@ icl_DumpKernel(FILE *outFilep, char *setname)
 			    i, (long)tname, sizeof(tname), 0, 0);
 	    if (code) {
 		if (errno == EBADF) {
-		    code = 0;
 		    continue;	/* missing slot, nothing to worry about */
 		}
 		break;
@@ -688,12 +640,10 @@ icl_DumpKernel(FILE *outFilep, char *setname)
 	    found++;
 	    if (dummy > bufferSize)	/* find biggest log */
 		bufferSize = dummy;
-	    lip = (struct logInfo *)malloc(sizeof(struct logInfo));
-	    memset((char *)lip, 0, sizeof(*lip));
+	    lip = calloc(1, sizeof(struct logInfo));
 	    lip->nextp = allInfo;
 	    allInfo = lip;
-	    lip->name = (char *)malloc(strlen(tname) + 1);
-	    strcpy(lip->name, tname);
+	    lip->name = strdup(tname);
 	}
 	i = found;
     } else {
@@ -706,18 +656,16 @@ icl_DumpKernel(FILE *outFilep, char *setname)
 		break;
 	    if (dummy > bufferSize)	/* find biggest log */
 		bufferSize = dummy;
-	    lip = (struct logInfo *)malloc(sizeof(struct logInfo));
-	    memset((char *)lip, 0, sizeof(*lip));
+	    lip = calloc(1, sizeof(struct logInfo));
 	    lip->nextp = allInfo;
 	    allInfo = lip;
-	    lip->name = (char *)malloc(strlen(tname) + 1);
-	    strcpy(lip->name, tname);
+	    lip->name = strdup(tname);
 	}
     }
 
     if (bufferSize == 0)
 	return -1;
-    bufferp = (afs_int32 *) malloc(sizeof(afs_int32) * bufferSize);
+    bufferp = malloc(sizeof(afs_int32) * bufferSize);
     if (!bufferp)
 	return -1;
 
@@ -778,10 +726,6 @@ icl_DumpKernel(FILE *outFilep, char *setname)
 	    /* print the record */
 	    DisplayRecord(outFilep, &bufferp[ix], rlength);
 	    ix += rlength;
-#ifdef notdef
-	    /* obsolete: read entire buffer first */
-	    i += rlength;	/* update cookie value, too */
-#endif
 	}			/* for loop displaying buffer */
     }				/* for loop over all logs */
 
@@ -1003,7 +947,7 @@ icl_TailKernel(FILE *outFilep, char *logname, afs_int32 waitTime)
 
     if (bufferSize == 0)
 	return -1;
-    bufferp = (afs_int32 *) malloc(sizeof(afs_int32) * bufferSize);
+    bufferp = malloc(sizeof(afs_int32) * bufferSize);
     if (!bufferp) {
 	(void)fprintf(stderr, "cannot allocate %d words for buffer\n",
 		      bufferSize);
@@ -1097,7 +1041,7 @@ icl_TailKernel(FILE *outFilep, char *logname, afs_int32 waitTime)
 	    /* have to reallocate a buffer */
 	    bufferSize = newBufferSize;
 	    free(bufferp);
-	    bufferp = (afs_int32 *) malloc(sizeof(afs_int32) * bufferSize);
+	    bufferp = malloc(sizeof(afs_int32) * bufferSize);
 	    if (!bufferp) {
 		(void)fprintf(stderr, "cannot allocate %d words for buffer\n",
 			      bufferSize);
@@ -1114,10 +1058,13 @@ icl_TailKernel(FILE *outFilep, char *logname, afs_int32 waitTime)
 
 #if !defined(AFS_SGI_ENV)
 int
-afs_syscall(long call, long parm0, long parm1, long parm2, long parm3, 
+afs_syscall(long call, long parm0, long parm1, long parm2, long parm3,
 	    long parm4, long parm5, long parm6)
 {
-    int code, rval;
+    int code;
+#if defined(AFS_DARWIN80_ENV) || defined(AFS_LINUX20_ENV)
+    int rval;
+#endif
 #ifdef AFS_LINUX20_ENV
 #if defined AFS_LINUX_64BIT_KERNEL
     long long eparm[4];
@@ -1135,13 +1082,23 @@ afs_syscall(long call, long parm0, long parm1, long parm2, long parm3,
 #endif
     /* Linux can only handle 5 arguments in the actual syscall. */
     if (call == AFSCALL_ICL) {
-	rval = proc_afs_syscall(call, parm0, parm1, parm2, eparm, &code);
-	if (rval)
+	rval = proc_afs_syscall(call, parm0, parm1, parm2, (long)eparm, &code);
+	if (rval) {
+#ifdef AFS_SYSCALL
 	    code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, eparm);
+#else
+	    code = -1;
+#endif
+	}
     } else {
 	rval = proc_afs_syscall(call, parm0, parm1, parm2, parm3, &code);
-	if (rval)
+	if (rval) {
+#ifdef AFS_SYSCALL
 	    code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, parm3);
+#else
+	    code = -1;
+#endif
+	}
     }
 #if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_SPARC_LINUX20_ENV)
     /* on sparc this function returns none value, so do it myself */
@@ -1153,7 +1110,11 @@ afs_syscall(long call, long parm0, long parm1, long parm2, long parm3,
     if (!code) code = rval;
 #else
 #if !defined(AFS_SGI_ENV) && !defined(AFS_AIX32_ENV)
+# if defined(AFS_SYSCALL)
     code = syscall(AFS_SYSCALL, call, parm0, parm1, parm2, parm3, parm4);
+# else
+    code = -1;
+# endif
 #else
 #if defined(AFS_SGI_ENV)
     code = syscall(AFS_ICL, call, parm0, parm1, parm2, parm3, parm4);	/* XXX */
@@ -1167,528 +1128,6 @@ afs_syscall(long call, long parm0, long parm1, long parm2, long parm3,
 }
 #endif
 
-
-int icl_inited = 0;
-
-/* init function, called once, under icl_lock */
-int
-icl_Init(void)
-{
-    icl_inited = 1;
-
-#ifndef KERNEL
-    /* setup signal handler, in user space */
-#endif /* KERNEL */
-
-    return 0;
-}
-
-int
-icl_CreateSet(char *name, struct afs_icl_log *baseLogp, 
-	      struct afs_icl_log *fatalLogp, struct afs_icl_set **outSetpp)
-{
-    return icl_CreateSetWithFlags(name, baseLogp, fatalLogp, /*flags */ 0,
-				  outSetpp);
-}
-
-/* create a set, given pointers to base and fatal logs, if any.
- * Logs are unlocked, but referenced, and *outSetpp is returned
- * referenced.  Function bumps reference count on logs, since it
- * addds references from the new icl_set.  When the set is destroyed,
- * those references will be released.
- */
-int
-icl_CreateSetWithFlags(char *name, struct afs_icl_log *baseLogp, 
-		       struct afs_icl_log *fatalLogp, afs_uint32 flags, 
-		       struct afs_icl_set **outSetpp)
-{
-    register struct afs_icl_set *setp;
-    register int i;
-    afs_int32 states = ICL_DEFAULT_SET_STATES;
-
-    if (!icl_inited)
-	icl_Init();
-
-    for (setp = icl_allSets; setp; setp = setp->nextp) {
-	if (strcmp(setp->name, name) == 0) {
-	    setp->refCount++;
-	    *outSetpp = setp;
-	    if (flags & ICL_CRSET_FLAG_PERSISTENT) {
-		setp->states |= ICL_SETF_PERSISTENT;
-	    }
-	    return 0;
-	}
-    }
-
-    /* determine initial state */
-    if (flags & ICL_CRSET_FLAG_DEFAULT_ON)
-	states = ICL_SETF_ACTIVE;
-    else if (flags & ICL_CRSET_FLAG_DEFAULT_OFF)
-	states = ICL_SETF_FREED;
-    if (flags & ICL_CRSET_FLAG_PERSISTENT)
-	states |= ICL_SETF_PERSISTENT;
-
-    setp = (struct afs_icl_set *)osi_Alloc(sizeof(struct afs_icl_set));
-    memset((caddr_t) setp, 0, sizeof(*setp));
-    setp->refCount = 1;
-    if (states & ICL_SETF_FREED)
-	states &= ~ICL_SETF_ACTIVE;	/* if freed, can't be active */
-    setp->states = states;
-
-    setp->name = (char *)osi_Alloc(strlen(name) + 1);
-    strcpy(setp->name, name);
-    setp->nevents = ICL_DEFAULTEVENTS;
-    setp->eventFlags = (char *)osi_Alloc(ICL_DEFAULTEVENTS);
-    for (i = 0; i < ICL_DEFAULTEVENTS; i++)
-	setp->eventFlags[i] = 0xff;	/* default to enabled */
-
-    /* update this global info under the icl_lock */
-    setp->nextp = icl_allSets;
-    icl_allSets = setp;
-
-    /* set's basic lock is still held, so we can finish init */
-    if (baseLogp) {
-	setp->logs[0] = baseLogp;
-	icl_LogHold(baseLogp);
-	if (!(setp->states & ICL_SETF_FREED))
-	    icl_LogUse(baseLogp);	/* log is actually being used */
-    }
-    if (fatalLogp) {
-	setp->logs[1] = fatalLogp;
-	icl_LogHold(fatalLogp);
-	if (!(setp->states & ICL_SETF_FREED))
-	    icl_LogUse(fatalLogp);	/* log is actually being used */
-    }
-
-    *outSetpp = setp;
-    return 0;
-}
-
-/* function to change event enabling information for a particular set */
-int
-icl_SetEnable(struct afs_icl_set *setp, afs_int32 eventID, int setValue)
-{
-    char *tp;
-
-    if (!ICL_EVENTOK(setp, eventID)) {
-	return -1;
-    }
-    tp = &setp->eventFlags[ICL_EVENTBYTE(eventID)];
-    if (setValue)
-	*tp |= ICL_EVENTMASK(eventID);
-    else
-	*tp &= ~(ICL_EVENTMASK(eventID));
-    return 0;
-}
-
-/* return indication of whether a particular event ID is enabled
- * for tracing.  If *getValuep is set to 0, the event is disabled,
- * otherwise it is enabled.  All events start out enabled by default.
- */
-int
-icl_GetEnable(struct afs_icl_set *setp, afs_int32 eventID, int *getValuep)
-{
-    if (!ICL_EVENTOK(setp, eventID)) {
-	return -1;
-    }
-    if (setp->eventFlags[ICL_EVENTBYTE(eventID)] & ICL_EVENTMASK(eventID))
-	*getValuep = 1;
-    else
-	*getValuep = 0;
-    return 0;
-}
-
-/* hold and release event sets */
-int
-icl_SetHold(register struct afs_icl_set *setp)
-{
-    setp->refCount++;
-    return 0;
-}
-
-/* free a set.  Called with icl_lock locked */
-int
-icl_ZapSet(register struct afs_icl_set *setp)
-{
-    register struct afs_icl_set **lpp, *tp;
-    int i;
-    register struct afs_icl_log *tlp;
-
-    for (lpp = &icl_allSets, tp = *lpp; tp; lpp = &tp->nextp, tp = *lpp) {
-	if (tp == setp) {
-	    /* found the dude we want to remove */
-	    *lpp = setp->nextp;
-	    osi_Free(setp->name, 1 + strlen(setp->name));
-	    osi_Free(setp->eventFlags, ICL_EVENTBYTES(setp->nevents));
-	    for (i = 0; i < ICL_LOGSPERSET; i++) {
-		if ((tlp = setp->logs[i]))
-		    icl_LogReleNL(tlp);
-	    }
-	    osi_Free(setp, sizeof(struct afs_icl_set));
-	    break;		/* won't find it twice */
-	}
-    }
-    return 0;
-}
-
-/* do the release, watching for deleted entries */
-int
-icl_SetRele(register struct afs_icl_set *setp)
-{
-    if (--setp->refCount == 0 && (setp->states & ICL_SETF_DELETED)) {
-	icl_ZapSet(setp);	/* destroys setp's lock! */
-    }
-    return 0;
-}
-
-/* free a set entry, dropping its reference count */
-int
-icl_SetFree(register struct afs_icl_set *setp)
-{
-    setp->states |= ICL_SETF_DELETED;
-    icl_SetRele(setp);
-    return 0;
-}
-
-/* find a set by name, returning it held */
-struct afs_icl_set *
-icl_FindSet(char *name)
-{
-    register struct afs_icl_set *tp;
-
-    for (tp = icl_allSets; tp; tp = tp->nextp) {
-	if (strcmp(tp->name, name) == 0) {
-	    /* this is the dude we want */
-	    tp->refCount++;
-	    break;
-	}
-    }
-    return tp;
-}
-
-/* zero out all the logs in the set */
-int
-icl_ZeroSet(struct afs_icl_set *setp)
-{
-    register int i;
-    int code = 0;
-    int tcode;
-    struct afs_icl_log *logp;
-
-    for (i = 0; i < ICL_LOGSPERSET; i++) {
-	logp = setp->logs[i];
-	if (logp) {
-	    icl_LogHold(logp);
-	    tcode = icl_ZeroLog(logp);
-	    if (tcode != 0)
-		code = tcode;	/* save the last bad one */
-	    icl_LogRele(logp);
-	}
-    }
-    return code;
-}
-
-int
-icl_EnumerateSets(int (*aproc) (char *, void *, struct afs_icl_set *), 
-		  void *arock)
-{
-    register struct afs_icl_set *tp, *np;
-    register afs_int32 code;
-
-    code = 0;
-    for (tp = icl_allSets; tp; tp = np) {
-	tp->refCount++;		/* hold this guy */
-	code = (*aproc) (tp->name, arock, tp);
-	np = tp->nextp;		/* tp may disappear next, but not np */
-	if (--tp->refCount == 0 && (tp->states & ICL_SETF_DELETED))
-	    icl_ZapSet(tp);
-	if (code)
-	    break;
-    }
-    return code;
-}
-
-int
-icl_AddLogToSet(struct afs_icl_set *setp, struct afs_icl_log *newlogp)
-{
-    register int i;
-    int code = -1;
-
-    for (i = 0; i < ICL_LOGSPERSET; i++) {
-	if (!setp->logs[i]) {
-	    setp->logs[i] = newlogp;
-	    code = i;
-	    icl_LogHold(newlogp);
-	    if (!(setp->states & ICL_SETF_FREED)) {
-		/* bump up the number of sets using the log */
-		icl_LogUse(newlogp);
-	    }
-	    break;
-	}
-    }
-    return code;
-}
-
-int
-icl_SetSetStat(struct afs_icl_set *setp, int op)
-{
-    int i;
-    afs_int32 code;
-    struct afs_icl_log *logp;
-
-    switch (op) {
-    case ICL_OP_SS_ACTIVATE:	/* activate a log */
-	/*
-	 * If we are not already active, see if we have released
-	 * our demand that the log be allocated (FREED set).  If
-	 * we have, reassert our desire.
-	 */
-	if (!(setp->states & ICL_SETF_ACTIVE)) {
-	    if (setp->states & ICL_SETF_FREED) {
-		/* have to reassert desire for logs */
-		for (i = 0; i < ICL_LOGSPERSET; i++) {
-		    logp = setp->logs[i];
-		    if (logp) {
-			icl_LogHold(logp);
-			icl_LogUse(logp);
-			icl_LogRele(logp);
-		    }
-		}
-		setp->states &= ~ICL_SETF_FREED;
-	    }
-	    setp->states |= ICL_SETF_ACTIVE;
-	}
-	code = 0;
-	break;
-
-    case ICL_OP_SS_DEACTIVATE:	/* deactivate a log */
-	/* this doesn't require anything beyond clearing the ACTIVE flag */
-	setp->states &= ~ICL_SETF_ACTIVE;
-	code = 0;
-	break;
-
-    case ICL_OP_SS_FREE:	/* deassert design for log */
-	/* 
-	 * if we are already in this state, do nothing; otherwise
-	 * deassert desire for log
-	 */
-	if (setp->states & ICL_SETF_ACTIVE)
-	    code = EINVAL;
-	else {
-	    if (!(setp->states & ICL_SETF_FREED)) {
-		for (i = 0; i < ICL_LOGSPERSET; i++) {
-		    logp = setp->logs[i];
-		    if (logp) {
-			icl_LogHold(logp);
-			icl_LogFreeUse(logp);
-			icl_LogRele(logp);
-		    }
-		}
-		setp->states |= ICL_SETF_FREED;
-	    }
-	    code = 0;
-	}
-	break;
-
-    default:
-	code = EINVAL;
-    }
-
-    return code;
-}
-
-struct afs_icl_log *afs_icl_allLogs = 0;
-
-/* hold and release logs */
-int
-icl_LogHold(register struct afs_icl_log *logp)
-{
-    logp->refCount++;
-    return 0;
-}
-
-/* hold and release logs, called with lock already held */
-int
-icl_LogHoldNL(register struct afs_icl_log *logp)
-{
-    logp->refCount++;
-    return 0;
-}
-
-/* keep track of how many sets believe the log itself is allocated */
-int
-icl_LogUse(register struct afs_icl_log *logp)
-{
-    if (logp->setCount == 0) {
-	/* this is the first set actually using the log -- allocate it */
-	if (logp->logSize == 0) {
-	    /* we weren't passed in a hint and it wasn't set */
-	    logp->logSize = ICL_DEFAULT_LOGSIZE;
-	}
-	logp->datap =
-	    (afs_int32 *) osi_Alloc(sizeof(afs_int32) * logp->logSize);
-    }
-    logp->setCount++;
-    return 0;
-}
-
-/* decrement the number of real users of the log, free if possible */
-int
-icl_LogFreeUse(register struct afs_icl_log *logp)
-{
-    if (--logp->setCount == 0) {
-	/* no more users -- free it (but keep log structure around) */
-	osi_Free(logp->datap, sizeof(afs_int32) * logp->logSize);
-	logp->firstUsed = logp->firstFree = 0;
-	logp->logElements = 0;
-	logp->datap = NULL;
-    }
-    return 0;
-}
-
-/* set the size of the log to 'logSize' */
-int
-icl_LogSetSize(register struct afs_icl_log *logp, afs_int32 logSize)
-{
-    if (!logp->datap) {
-	/* nothing to worry about since it's not allocated */
-	logp->logSize = logSize;
-    } else {
-	/* reset log */
-	logp->firstUsed = logp->firstFree = 0;
-	logp->logElements = 0;
-
-	/* free and allocate a new one */
-	osi_Free(logp->datap, sizeof(afs_int32) * logp->logSize);
-	logp->datap = (afs_int32 *) osi_Alloc(sizeof(afs_int32) * logSize);
-	logp->logSize = logSize;
-    }
-
-    return 0;
-}
-
-/* free a log.  Called with icl_lock locked. */
-int
-icl_ZapLog(register struct afs_icl_log *logp)
-{
-    register struct afs_icl_log **lpp, *tp;
-
-    for (lpp = &afs_icl_allLogs, tp = *lpp; tp; lpp = &tp->nextp, tp = *lpp) {
-	if (tp == logp) {
-	    /* found the dude we want to remove */
-	    *lpp = logp->nextp;
-	    osi_Free(logp->name, 1 + strlen(logp->name));
-	    osi_Free(logp->datap, logp->logSize * sizeof(afs_int32));
-	    osi_Free(logp, sizeof(struct icl_log));
-	    break;		/* won't find it twice */
-	}
-    }
-    return 0;
-}
-
-/* do the release, watching for deleted entries */
-int
-icl_LogRele(register struct afs_icl_log *logp)
-{
-    if (--logp->refCount == 0 && (logp->states & ICL_LOGF_DELETED)) {
-	icl_ZapLog(logp);	/* destroys logp's lock! */
-    }
-    return 0;
-}
-
-/* do the release, watching for deleted entries, log already held */
-int
-icl_LogReleNL(register struct afs_icl_log *logp)
-{
-    if (--logp->refCount == 0 && (logp->states & ICL_LOGF_DELETED)) {
-	icl_ZapLog(logp);	/* destroys logp's lock! */
-    }
-    return 0;
-}
-
-/* zero out the log */
-int
-icl_ZeroLog(register struct afs_icl_log *logp)
-{
-    logp->firstUsed = logp->firstFree = 0;
-    logp->logElements = 0;
-    return 0;
-}
-
-/* free a log entry, and drop its reference count */
-int
-icl_LogFree(register struct afs_icl_log *logp)
-{
-    logp->states |= ICL_LOGF_DELETED;
-    icl_LogRele(logp);
-    return 0;
-}
-
-
-int
-icl_EnumerateLogs(int (*aproc)
-		    (char *name, void *arock, struct afs_icl_log * tp),
-		  void *arock)
-{
-    register struct afs_icl_log *tp;
-    register afs_int32 code;
-
-    code = 0;
-    for (tp = afs_icl_allLogs; tp; tp = tp->nextp) {
-	tp->refCount++;		/* hold this guy */
-	code = (*aproc) (tp->name, arock, tp);
-	if (--tp->refCount == 0)
-	    icl_ZapLog(tp);
-	if (code)
-	    break;
-    }
-    return code;
-}
-
-
-afs_icl_bulkSetinfo_t *
-GetBulkSetInfo(void)
-{
-    unsigned int infoSize;
-
-    infoSize =
-	sizeof(afs_icl_bulkSetinfo_t) + (ICL_RPC_MAX_SETS -
-					 1) * sizeof(afs_icl_setinfo_t);
-    if (!setInfo) {
-	setInfo = (afs_icl_bulkSetinfo_t *) malloc(infoSize);
-	if (!setInfo) {
-	    (void)fprintf(stderr,
-			  "Could not allocate the memory for bulk set info structure\n");
-	    exit(1);
-	}
-    }
-    memset((char *)setInfo, 0, infoSize);
-
-    return setInfo;
-}
-
-afs_icl_bulkLoginfo_t *
-GetBulkLogInfo(void)
-{
-    unsigned int infoSize;
-
-    infoSize =
-	sizeof(afs_icl_bulkLoginfo_t) + (ICL_RPC_MAX_LOGS -
-					 1) * sizeof(afs_icl_loginfo_t);
-    if (!logInfo) {
-	logInfo = (afs_icl_bulkLoginfo_t *) malloc(infoSize);
-	if (!logInfo) {
-	    (void)fprintf(stderr,
-			  "Could not allocate the memory for bulk log info structure\n");
-	    exit(1);
-	}
-    }
-
-    memset((char *)logInfo, 0, infoSize);
-    return logInfo;
-}
-
-
 static int
 DoDump(struct cmd_syndesc *as, void *arock)
 {
@@ -1764,18 +1203,20 @@ SetUpDump(void)
     struct cmd_syndesc *dumpSyntax;
 
     dumpSyntax =
-	cmd_CreateSyntax("dump", DoDump, NULL, "dump AFS trace logs");
-    (void)cmd_AddParm(dumpSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
+	cmd_CreateSyntax("dump", DoDump, NULL, 0, "dump AFS trace logs");
+    (void)cmd_AddParm(dumpSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
+		      "event set name");
     (void)cmd_AddParm(dumpSyntax, "-follow", CMD_SINGLE, CMD_OPTIONAL,
-		      "log_name");
+		      "trace log name");
     (void)cmd_AddParm(dumpSyntax, "-file", CMD_SINGLE, CMD_OPTIONAL,
-		      "output_filename");
+		      "path to trace log file for writing");
     (void)cmd_AddParm(dumpSyntax, "-sleep", CMD_SINGLE, CMD_OPTIONAL,
-		      "seconds_between_reads");
+		      "interval (secs) for writes when using -follow");
 }
 
+
 static int
-DoShowLog(register struct cmd_syndesc *as, void *arock)
+DoShowLog(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -1838,15 +1279,18 @@ SetUpShowLog(void)
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lslog", DoShowLog, NULL,
+	cmd_CreateSyntax("lslog", DoShowLog, NULL, 0,
 			 "list available logs");
-    (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
-    (void)cmd_AddParm(showSyntax, "-log", CMD_LIST, CMD_OPTIONAL, "log_name");
-    (void)cmd_AddParm(showSyntax, "-long", CMD_FLAG, CMD_OPTIONAL, "");
+    (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
+		      "event set name");
+    (void)cmd_AddParm(showSyntax, "-log", CMD_LIST, CMD_OPTIONAL,
+		      "trace log name");
+    (void)cmd_AddParm(showSyntax, "-long", CMD_FLAG, CMD_OPTIONAL,
+		      "show defined log size in kbytes & if it is allocated in kernel mem");
 }
 
 static int
-DoShowSet(register struct cmd_syndesc *as, void *arock)
+DoShowSet(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -1894,13 +1338,14 @@ SetUpShowSet(void)
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lsset", DoShowSet, NULL,
+	cmd_CreateSyntax("lsset", DoShowSet, NULL, 0,
 			 "list available event sets");
-    (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
+    (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
+		      "event set name");
 }
 
 static int
-DoClear(register struct cmd_syndesc *as, void *arock)
+DoClear(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -1951,16 +1396,16 @@ SetUpClear(void)
     struct cmd_syndesc *clearSyntax;
 
     clearSyntax =
-	cmd_CreateSyntax("clear", DoClear, NULL,
+	cmd_CreateSyntax("clear", DoClear, NULL, 0,
 			 "clear logs by logname or by event set");
     (void)cmd_AddParm(clearSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
-		      "set_name");
+		      "event set name");
     (void)cmd_AddParm(clearSyntax, "-log", CMD_LIST, CMD_OPTIONAL,
-		      "log_name");
+		      "trace log name");
 }
 
 static int
-DoSet(register struct cmd_syndesc *as, void *arock)
+DoSet(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2038,16 +1483,20 @@ SetUpSet(void)
     struct cmd_syndesc *setSyntax;
 
     setSyntax =
-	cmd_CreateSyntax("setset", DoSet, NULL,
+	cmd_CreateSyntax("setset", DoSet, NULL, 0,
 			 "set state of event sets");
-    (void)cmd_AddParm(setSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
-    (void)cmd_AddParm(setSyntax, "-active", CMD_FLAG, CMD_OPTIONAL, "");
-    (void)cmd_AddParm(setSyntax, "-inactive", CMD_FLAG, CMD_OPTIONAL, "");
-    (void)cmd_AddParm(setSyntax, "-dormant", CMD_FLAG, CMD_OPTIONAL, "");
+    (void)cmd_AddParm(setSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
+		      "event set name");
+    (void)cmd_AddParm(setSyntax, "-active", CMD_FLAG, CMD_OPTIONAL,
+		      "enable tracing for event set & allocate kernel memory");
+    (void)cmd_AddParm(setSyntax, "-inactive", CMD_FLAG, CMD_OPTIONAL,
+		      "disables tracing for event set, keep kernel memory");
+    (void)cmd_AddParm(setSyntax, "-dormant", CMD_FLAG, CMD_OPTIONAL,
+		      "disable tracing for event set & free kernel memory");
 }
 
 static int
-DoResize(register struct cmd_syndesc *as, void *arock)
+DoResize(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2095,12 +1544,12 @@ SetUpResize(void)
     struct cmd_syndesc *setsizeSyntax;
 
     setsizeSyntax =
-	cmd_CreateSyntax("setlog", DoResize, NULL,
+	cmd_CreateSyntax("setlog", DoResize, NULL, 0,
 			 "set the size of a log");
     (void)cmd_AddParm(setsizeSyntax, "-log", CMD_LIST, CMD_OPTIONAL,
-		      "log_name");
+		      "trace log name");
     (void)cmd_AddParm(setsizeSyntax, "-buffersize", CMD_SINGLE, CMD_REQUIRED,
-		      "1-kilobyte_units");
+		      "# of 1-kbyte blocks to allocate for log");
 }
 
 #include "AFS_component_version_number.c"
@@ -2123,12 +1572,3 @@ main(int argc, char *argv[])
 
     return (cmd_Dispatch(argc, argv));
 }
-#else
-#include "AFS_component_version_number.c"
-
-int
-main(int argc, char *argv[])
-{
-    printf("fstrace is NOT supported for this OS\n");
-}
-#endif
