@@ -21,7 +21,6 @@
 
 struct vlheader xheader;
 extern int maxnservers;
-struct extentaddr extentaddr;
 extern afs_uint32 rd_HostAddress[MAXSERVERID + 1];
 extern afs_uint32 wr_HostAddress[MAXSERVERID + 1];
 struct extentaddr *rd_ex_addr[VL_MAX_ADDREXTBLKS] = { 0, 0, 0, 0 };
@@ -179,9 +178,15 @@ vlentryread(struct ubik_trans *trans, afs_int32 offset, char *buffer,
 	    nbufp->nextIdHash[i] = ntohl(oep->nextIdHash[i]);
 	nbufp->nextNameHash = ntohl(oep->nextNameHash);
 	memcpy(nbufp->name, oep->name, VL_MAXNAMELEN);
-	memcpy(nbufp->serverNumber, oep->serverNumber, NMAXNSERVERS);
-	memcpy(nbufp->serverPartition, oep->serverPartition, NMAXNSERVERS);
-	memcpy(nbufp->serverFlags, oep->serverFlags, NMAXNSERVERS);
+	memcpy(nbufp->serverNumber, oep->serverNumber, OMAXNSERVERS);
+	memcpy(nbufp->serverPartition, oep->serverPartition, OMAXNSERVERS);
+	memcpy(nbufp->serverFlags, oep->serverFlags, OMAXNSERVERS);
+	/* initilize the last elements to BADSERVERID */
+	for (i = OMAXNSERVERS; i < NMAXNSERVERS; i++) {
+	    nbufp->serverNumber[i] = BADSERVERID;
+	    nbufp->serverPartition[i] = BADSERVERID;
+	    nbufp->serverFlags[i] = BADSERVERID;
+	}
     }
     return 0;
 }
@@ -221,7 +226,7 @@ readExtents(struct ubik_trans *trans)
 
     /* Read the first extension block */
     if (!rd_ex_addr[0]) {
-	rd_ex_addr[0] = (struct extentaddr *)malloc(VL_ADDREXTBLK_SIZE);
+	rd_ex_addr[0] = malloc(VL_ADDREXTBLK_SIZE);
 	if (!rd_ex_addr[0])
 	    ERROR_EXIT(VL_NOMEM);
     }
@@ -252,7 +257,7 @@ readExtents(struct ubik_trans *trans)
 
 	/* Read the continuation block */
 	if (!rd_ex_addr[i]) {
-	    rd_ex_addr[i] = (struct extentaddr *)malloc(VL_ADDREXTBLK_SIZE);
+	    rd_ex_addr[i] = malloc(VL_ADDREXTBLK_SIZE);
 	    if (!rd_ex_addr[i])
 		ERROR_EXIT(VL_NOMEM);
 	}
@@ -326,36 +331,39 @@ UpdateCache(struct ubik_trans *trans, void *rock)
     /* now, if can't read, or header is wrong, write a new header */
     if (ubcode || vldbversion == 0) {
 	if (builddb) {
-	    printf("Can't read VLDB header, re-initialising...\n");
+	    VLog(0, ("Can't read VLDB header, re-initialising...\n"));
 
 	    /* try to write a good header */
-	    memset(&rd_cheader, 0, sizeof(rd_cheader));
-	    rd_cheader.vital_header.vldbversion = htonl(VLDBVERSION);
-	    rd_cheader.vital_header.headersize = htonl(sizeof(rd_cheader));
+	    /* The read cache will be sync'ed to this new header
+	     * when the ubik transaction is ended by vlsynccache(). */
+	    memset(&wr_cheader, 0, sizeof(wr_cheader));
+	    wr_cheader.vital_header.vldbversion = htonl(VLDBVERSION);
+	    wr_cheader.vital_header.headersize = htonl(sizeof(wr_cheader));
 	    /* DANGER: Must get this from a master place!! */
-	    rd_cheader.vital_header.MaxVolumeId = htonl(0x20000000);
-	    rd_cheader.vital_header.eofPtr = htonl(sizeof(rd_cheader));
+	    wr_cheader.vital_header.MaxVolumeId = htonl(0x20000000);
+	    wr_cheader.vital_header.eofPtr = htonl(sizeof(wr_cheader));
 	    for (i = 0; i < MAXSERVERID + 1; i++) {
-		rd_cheader.IpMappedAddr[i] = 0;
-		rd_HostAddress[i] = 0;
+		wr_cheader.IpMappedAddr[i] = 0;
+		wr_HostAddress[i] = 0;
 	    }
-	    code = vlwrite(trans, 0, (char *)&rd_cheader, sizeof(rd_cheader));
+	    code = vlwrite(trans, 0, (char *)&wr_cheader, sizeof(wr_cheader));
 	    if (code) {
-		printf("Can't write VLDB header (error = %d)\n", code);
+		VLog(0, ("Can't write VLDB header (error = %d)\n", code));
 		ERROR_EXIT(VL_IO);
 	    }
-	    vldbversion = ntohl(rd_cheader.vital_header.vldbversion);
+	    vldbversion = ntohl(wr_cheader.vital_header.vldbversion);
 	} else {
+	    VLog(1, ("Unable to read VLDB header.\n"));
 	    ERROR_EXIT(VL_EMPTY);
 	}
     }
 
     if ((vldbversion != VLDBVERSION) && (vldbversion != OVLDBVERSION)
         && (vldbversion != VLDBVERSION_4)) {
-	printf
+	VLog(0,
 	    ("VLDB version %d doesn't match this software version(%d, %d or %d), quitting!\n",
-	     vldbversion, VLDBVERSION_4, VLDBVERSION, OVLDBVERSION);
-	return VL_BADVERSION;
+	     vldbversion, VLDBVERSION_4, VLDBVERSION, OVLDBVERSION));
+	ERROR_EXIT(VL_BADVERSION);
     }
 
     maxnservers = ((vldbversion == 3 || vldbversion == 4) ? 13 : 8);
@@ -389,9 +397,34 @@ CheckInit(struct ubik_trans *trans, int builddb)
     return 0;
 }
 
+/**
+ * Grow the eofPtr in the header by 'bump' bytes.
+ *
+ * @param[inout] cheader    VL header
+ * @param[in] bump	    How many bytes to add to eofPtr
+ * @param[out] a_blockindex On success, set to the original eofPtr before we
+ *			    bumped it
+ * @return VL error codes
+ */
+static afs_int32
+grow_eofPtr(struct vlheader *cheader, afs_int32 bump, afs_int32 *a_blockindex)
+{
+    afs_int32 blockindex = ntohl(cheader->vital_header.eofPtr);
+
+    if (blockindex < 0 || blockindex >= MAX_AFS_INT32 - bump) {
+	VLog(0, ("Error: Tried to grow the VLDB beyond the 2GiB limit. Either "
+		 "find a way to trim down your VLDB, or upgrade to a release "
+		 "and database format that supports a larger VLDB.\n"));
+	return VL_IO;
+    }
+
+    *a_blockindex = blockindex;
+    cheader->vital_header.eofPtr = htonl(blockindex + bump);
+    return 0;
+}
 
 afs_int32
-GetExtentBlock(struct vl_ctx *ctx, register afs_int32 base)
+GetExtentBlock(struct vl_ctx *ctx, afs_int32 base)
 {
     afs_int32 blockindex, code, error = 0;
 
@@ -402,7 +435,7 @@ GetExtentBlock(struct vl_ctx *ctx, register afs_int32 base)
     if (!ctx->ex_addr[0] || !ctx->ex_addr[0]->ex_contaddrs[base]) {
 	/* Create a new extension block */
 	if (!ctx->ex_addr[base]) {
-	    ctx->ex_addr[base] = (struct extentaddr *)malloc(VL_ADDREXTBLK_SIZE);
+	    ctx->ex_addr[base] = malloc(VL_ADDREXTBLK_SIZE);
 	    if (!ctx->ex_addr[base])
 		ERROR_EXIT(VL_NOMEM);
 	}
@@ -410,16 +443,17 @@ GetExtentBlock(struct vl_ctx *ctx, register afs_int32 base)
 
 	/* Write the full extension block at end of vldb */
 	ctx->ex_addr[base]->ex_hdrflags = htonl(VLCONTBLOCK);
-	blockindex = ntohl(ctx->cheader->vital_header.eofPtr);
+	code = grow_eofPtr(ctx->cheader, VL_ADDREXTBLK_SIZE, &blockindex);
+	if (code)
+	    ERROR_EXIT(VL_IO);
+
 	code =
 	    vlwrite(ctx->trans, blockindex, (char *)ctx->ex_addr[base],
 		    VL_ADDREXTBLK_SIZE);
 	if (code)
 	    ERROR_EXIT(VL_IO);
 
-	/* Update the cheader.vitalheader structure on disk */
-	ctx->cheader->vital_header.eofPtr = blockindex + VL_ADDREXTBLK_SIZE;
-	ctx->cheader->vital_header.eofPtr = htonl(ctx->cheader->vital_header.eofPtr);
+
 	code = write_vital_vlheader(ctx);
 	if (code)
 	    ERROR_EXIT(VL_IO);
@@ -564,9 +598,11 @@ AllocBlock(struct vl_ctx *ctx, struct nvlentry *tentry)
 	    return 0;
 	ctx->cheader->vital_header.freePtr = htonl(tentry->nextIdHash[0]);
     } else {
+	afs_int32 code;
 	/* hosed, nothing on free list, grow file */
-	blockindex = ntohl(ctx->cheader->vital_header.eofPtr);	/* remember this guy */
-	ctx->cheader->vital_header.eofPtr = htonl(blockindex + sizeof(vlentry));
+	code = grow_eofPtr(ctx->cheader, sizeof(vlentry), &blockindex);
+	if (code)
+	    return 0;
     }
     ctx->cheader->vital_header.allocs++;
     if (write_vital_vlheader(ctx))
@@ -858,13 +894,30 @@ UnthreadVLentry(struct vl_ctx *ctx, afs_int32 blockindex,
 
     /* Take the RO/RW entries of their respective hash linked lists. */
     for (typeindex = ROVOL; typeindex <= BACKVOL; typeindex++) {
-	if ((errorcode = UnhashVolid(ctx, typeindex, blockindex, aentry)))
+	if ((errorcode = UnhashVolid(ctx, typeindex, blockindex, aentry))) {
+	    if (errorcode == VL_NOENT) {
+		VLog(0, ("Unable to unhash vlentry '%s' (address %d) from hash "
+			 "chain for volid %u (type %d).\n",
+			 aentry->name, blockindex, aentry->volumeId[typeindex], typeindex));
+		VLog(0, ("... The VLDB may be partly corrupted; see vldb_check "
+			 "for how to check for and fix errors.\n"));
+		return VL_DBBAD;
+	    }
 	    return errorcode;
+	}
     }
 
     /* Take it out of the Volname hash list */
-    if ((errorcode = UnhashVolname(ctx, blockindex, aentry)))
+    if ((errorcode = UnhashVolname(ctx, blockindex, aentry))) {
+	if (errorcode == VL_NOENT) {
+	    VLog(0, ("Unable to unhash vlentry '%s' (address %d) from name "
+		     "hash chain.\n", aentry->name, blockindex));
+	    VLog(0, ("... The VLDB may be partly corrupted; see vldb_check "
+		     "for how to check for and fix errors.\n"));
+	    return VL_DBBAD;
+	}
 	return errorcode;
+    }
 
     /* Update cheader entry */
     write_vital_vlheader(ctx);
